import logging

from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *
from cumin.stat import *
from cumin.widgets import *
from cumin.parameters import *
from cumin.formats import *
from cumin.util import *

from job import *

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.limit")

class LimitDataSet(object):
    def __init__(self, app, collector):
        self.app = app
        self.collector = collector

    def get_negotiator(self, session):
        collector = self.collector.get(session)
        cls = self.app.model.com_redhat_grid.Negotiator

        # TODO: why is there more than 1 negotiator?
        negs = cls.get_selection(session.cursor, Pool=collector.Pool)
        try:
            youngest = negs[0]
        except IndexError:
            return None

        for neg in negs:
            if neg._qmf_update_time > youngest._qmf_update_time:
                youngest = neg
        negotiator = youngest

        return negotiator

    def get_items(self, session):
        negotiator = self.get_negotiator(session)

        if not negotiator:
            return

        limits = self.app.model.get_negotiator_limits(negotiator)

        if limits is None or len(limits) == 0:
            return ()
            #limits = {'a':{'CURRENT': 1, 'MAX': 2}}

        keys = limits.keys()
        keys.sort()

        return [{"name": x,
                 "curr": limits[x]["CURRENT"],
                 "max": limits[x]["MAX"]}
                for x in keys]

class LimitSet(CuminItemTable):
    def __init__(self, app, name, collector):
        super(LimitSet, self).__init__(app, name)

        self.data = LimitDataSet(app, collector)

        self.defer_enabled = True

        col = self.NameColumn(app, "name")
        self.add_column(col)
        self.set_default_column(col)

        col = self.CurrentColumn(app, "curr")
        col.align = "right"
        self.add_column(col)

        col = self.MaxColumn(app, "max")
        col.align = "right"
        self.add_column(col)

    def do_get_items(self, session):
        return self.data.get_items(session)

    def render_title(self, session):
        return "Limits"

    class NameColumn(NonSortableTableColumn):
        def render_title(self, session):
            return "Name"

        def render_content(self, session, data):
            limit_name = data["name"]
            try:
                limit_max = int(data["max"])
            except:
                limit_max = "0"
            negotiator = self.parent.data.get_negotiator(session)
            self.frame.limit.id.set(session, negotiator._id)
            self.frame.limit.set_limit.form.limit_name.set(session, limit_name)
            self.frame.limit.set_limit.form.limit_max.set(session, limit_max)
            href = self.frame.limit.set_limit.get_href(session)
            return fmt_link(href, limit_name)

    class CurrentColumn(NonSortableTableColumn):
        def render_title(self, session):
            return "Current Usage"

    class MaxColumn(NonSortableTableColumn):
        def render_title(self, session):
            return "Max Allowance"

        def render_content(self, session, data):
            try:
                limit_max = int(data["max"])
            except:
                limit_max = "0"
            return limit_max

    class Limits(Attribute):
        def get_default(self, session):
            return dict()

class LimitFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Negotiator

        super(LimitFrame, self).__init__(app, name, cls)

        self.set_limit = NegotiatorLimitTask(app, self)

class NegotiatorLimitTask(ObjectFrameTask):
    def __init__(self, app, frame):
        super(NegotiatorLimitTask, self).__init__(app, frame)

        self.form = NegotiatorLimitForm(app, self.name, self)

    def get_title(self, session):
        return "Set Limit"

    def do_enter(self, session, osession):
        self.form.limit_name.set(session, self.form.limit_name.get(osession))
        self.form.limit_max.set(session, self.form.limit_max.get(osession))

    def do_invoke(self, invoc, negotiator, limit_name, limit_max):
        action = QmfCall(self.app)
        limit_max = float(limit_max)
        results = action.execute(negotiator, "SetLimit", limit_name, limit_max)

        if results.error:
            raise results.error

        invoc.status_code = results.status

        action = QmfCall(self.app)
        action.execute(negotiator, "Reconfig")

        invoc.end()

class NegotiatorLimitForm(ObjectFrameTaskForm):
    def __init__(self, app, name, task):
        super(NegotiatorLimitForm, self).__init__(app, name, task)

        self.limit_name = self.LimitName(app, "name")
        self.add_field(self.limit_name)

        self.limit_max = self.LimitMax(app, "max")
        self.limit_max.required = True
        self.add_field(self.limit_max)

    def process_submit(self, session):
        self.validate(session)

        if not self.errors.get(session):
            limit_name = self.limit_name.get(session)
            limit_max = self.limit_max.get(session)
            negotiator = self.object.get(session)

            self.task.invoke(session, negotiator, limit_name, limit_max)
            self.task.exit_with_redirect(session)

    class LimitName(StringField):
        def __init__(self, app, name):
            super(NegotiatorLimitForm.LimitName, self).__init__(app, name)

            self.input = self.DisabledInput(app, "input")
            self.replace_child(self.input)

        def render_title(self, session):
            return "Limit name"

        class DisabledInput(StringInput):
            # used to override html and css
            pass

    class LimitMax(IntegerField):
        def render_title(self, session):
            return "Max Allowance"

class LimitView(CuminView):
    def __init__(self, app, name, limit):
        super(LimitView, self).__init__(app, name, None)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)
