import logging

from wooly import *
from wooly.forms import *
from wooly.resources import *
from wooly.sql import *
from wooly.tables import *
from wooly.widgets import *

from cumin.objectframe import *
from cumin.objectselector import *
from cumin.stat import *
from cumin.widgets import *
from cumin.parameters import *
from cumin.formats import *
from cumin.util import *

from collector import *
from job import *
from limit import *
from negotiator import *
from scheduler import *
from slot import *
from submission import *
from submitter import *

from cumin.widgets import Session

import cumin.model
import main

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.pool")

class PoolSelector(ObjectSelector):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector

        super(PoolSelector, self).__init__(app, name, cls)

        frame = "main.grid.pool"
        col = ObjectLinkColumn(app, "name", cls.Pool, cls._id, frame)
        self.add_column(col)

        self.add_attribute_column(cls.RunningJobs)
        self.add_attribute_column(cls.IdleJobs)
        self.add_attribute_column(cls.HostsTotal)

    def init(self):
        super(PoolSelector, self).init()

        table = self.cls.sql_table

        when = "now() - interval '1 day'"
        filter = SqlComparisonFilter(table._qmf_update_time, when, ">")
        self.adapter.query.add_filter(filter)

class PoolFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector

        super(PoolFrame, self).__init__(app, name, cls)

        self.icon_href = "resource?name=pool-36.png"

        self.submission = SubmissionFrame(app, "submission")
        self.add_mode(self.submission)

        self.slot = SlotFrame(app, "slot")
        self.add_mode(self.slot)

        self.scheduler = SchedulerFrame(app, "scheduler", self.object)
        self.add_mode(self.scheduler)

        self.negotiator = NegotiatorFrame(app, "negotiator")
        self.add_mode(self.negotiator)

        self.limit = LimitFrame(app, "limit")
        self.add_mode(self.limit)

        overview = PoolOverview(app, "overview", self.object)
        self.view.add_tab(overview)

        submissions = PoolSubmissionSelector(app, "submissions", self.object)
        self.view.add_tab(submissions)

        slots = SlotSelector(app, "slots", self.object)
        self.view.add_tab(slots)

        schedulers = SchedulerSelector(app, "schedulers", self.object)
        self.view.add_tab(schedulers)

        negotiators = NegotiatorSelector(app, "negotiators", self.object)
        self.view.add_tab(negotiators)

        self.negotiator_attribute = self.NegotiatorAttribute(app, "neg", self.object)
        self.add_attribute(self.negotiator_attribute)

        self.edit_dynamic_quota = NegotiatorEditDynamicQuota(app, self)
        self.quotas = NegotiatorOverview(app, "quotas", self.negotiator_attribute, self)
        self.view.add_tab(self.quotas)

        self.limits = LimitSet(app, "limits", self.object)
        self.view.add_tab(self.limits)

    class NegotiatorAttribute(Attribute):
        def __init__(self, app, name, collector):
            super(PoolFrame.NegotiatorAttribute, self).__init__(app, name)

            self.collector = collector

        def get(self, session):
            negotiator = super(PoolFrame.NegotiatorAttribute, self).get(session)
            if not negotiator:
                cls = self.app.model.com_redhat_grid.Negotiator

                pool = self.collector.get(session)
                if pool:
                    negotiators = cls.get_selection(session.cursor, Pool=pool.Pool)
                    # use the most recently updated one if there are multiple
                    if len(negotiators) > 0:
                        recent_negotiator = negotiators[0]
                        for neg in negotiators:
                            if neg._qmf_update_time > \
                                recent_negotiator._qmf_update_time:
                                recent_negotiator = neg
                        self.set(session, recent_negotiator)
                        negotiator = recent_negotiator
            return negotiator

class PoolOverview(Widget):
    def __init__(self, app, name, collector):
        super(PoolOverview, self).__init__(app, name)

        self.collector = collector

        self.grid = CollectorGridAttribute(app, "grid", self.collector)
        self.add_attribute(self.grid)

        stats = CollectorGeneralStatSet(app, "collector_stats", self.collector)
        self.add_child(stats)

        stats = GridStats(app, "grid_stats", self.grid)
        self.add_child(stats)

        slot_map = PoolSlotMap(app, "slot_png", self.collector)
        self.add_child(slot_map)

        slot_vis = PoolSlotFlashVis(app, "slot_map", self.collector)
        self.add_child(slot_vis)

        chart = self.JobStackedChart(app, "jobs", self.collector)
        chart.duration.param.default = "3600"
        chart.stats = ("RunningJobs", "IdleJobs")
        chart.chart_type = "stacked"
        self.add_child(chart)

        chart = self.SlotFlashChart(app, "slots", self.collector)
        chart.stats = ("HostsClaimed", "HostsUnclaimed", "HostsOwner")
        chart.chart_type = "stacked"
        self.add_child(chart)

    def render_title(self, session):
        return "Overview"

    def render_pool_name(self, session):
        collector = self.collector.get(session)
        return collector.Name

    class JobStackedChart(StatFlashChart):
        def render_title(self, session):
            return "Job status"

    class SlotFlashChart(StatFlashChart):
        def render_title(self, session):
            return "Slot state"

class GridGeneralStatSet(StatSet):
    def __init__(self, app, name, object):
        super(GridGeneralStatSet, self).__init__(app, name, object)

        self.attrs = ("NumJobs", "SubmitsInProgress",
                      "SubmitsQueued", "SubmitsAllowed",
                      "SubmitsWanted", "RunningJobs", "IdleJobs")

    def do_render(self, session):
        grid = self.object.get(session)

        if grid:
            return super(GridGeneralStatSet, self).do_render(session)

class GridStats(Widget):
    def __init__(self, app, name, grid):
        super(GridStats, self).__init__(app, name)

        self.grid = grid

        stats = GridGeneralStatSet(app, "stats", self.grid)
        self.add_child(stats)

        chart = self.GridJobChart(app, "jobs_chart", self.grid)
        chart.stats = ("NumJobs", "RunningJobs", "IdleJobs")
        self.add_child(chart)

        chart = self.GridSubmitChart(app, "submit_chart", self.grid)
        chart.stats = ("SubmitsAllowed", "SubmitsQueued", "SubmitsInProgress")
        self.add_child(chart)

    def render_grid_name(self, session):
        grid = self.grid.get(session)

        if grid:
            return grid.Name

    def do_render(self, session):
        grid = self.grid.get(session)

        if grid:
            return super(GridStats, self).do_render(session)

    class GridJobChart(StatFlashChart):
        def render_title(self, session):
            return "Grid jobs"

    class GridSubmitChart(StatFlashChart):
        def render_title(self, session):
            return "Grid submits"

class PoolSlotMap(SlotMap):
    def __init__(self, app, name, collector):
        super(PoolSlotMap, self).__init__(app, name)

        self.collector = collector

    def do_process(self, session):
        super(PoolSlotMap, self).do_process(session)

        collector = self.collector.get(session)

        self.slots.add_where_expr(session, "\"Pool\" = '%s'", collector.Pool)

    def render_image_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    def get_scheduler_select(self, session):
        pass
        #pool = self.pool.get(session)

class PoolSlotMapPage(SlotMapPage):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Collector
        self.collector = RosemaryObjectParameter(app, "id", cls)

        super(PoolSlotMapPage, self).__init__(app, name, self.collector, "Pool")

        self.add_parameter(self.collector)

    def do_process(self, session):
        super(PoolSlotMapPage, self).do_process(session)

        collector = self.collector.get(session)

        # XXX The conditional is necessary because this page is
        # overloaded to generate little dots, in which case it has no
        # pool

        if collector:
            self.slots.add_where_expr(session, "\"Pool\" = '%s'", collector.Pool)

class PoolSlotFlashVis(PoolSlotMap):
    def __init__(self, app, name, object):
        super(PoolSlotFlashVis, self).__init__(app, name, object)

        self.group_by = self.GroupBySwitch(app, "group_by")
        self.add_child(self.group_by)

        self.fullpageable = True

    def render_slots_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)
        page.json.set(sess, "slots")
        page.groups.set(sess, [self.group_by.get(session)])

        return sess.marshal()

    def render_image_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_page
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    def render_slot_chart_width(self, session):
        return self.render_slot_clip_size(session)

    def render_slot_chart_height(self, session):
        return self.render_slot_clip_size(session)

    def render_fullpageable(self, session):
        return self.fullpageable and "fullpageable" or ""

    def render_fullpage_href(self, session):
        collector = self.collector.get(session)

        page = self.app.grid.pool_slots_fullpage
        sess = Session(page)

        page.collector.set(sess, collector)

        return sess.marshal()

    class GroupBySwitch(StateSwitch):
        def __init__(self, app, name):
            super(PoolSlotFlashVis.GroupBySwitch, self).__init__(app, name)

            self.add_state("System", "By system")
            self.add_state("AccountingGroup", "By accounting group")
            self.add_state("None", "No Grouping")

        def get_click(self, session, state):
            href = self.parent.render_slots_href(session)
            return "return slot_vis('%s', this, '%s', '%s')" % \
                (state, self.parent.path, href)

class PoolSlotFullPage(FlashFullPage):
    def __init__(self, app, name):
        super(PoolSlotFullPage, self).__init__(app, name)

        cls = app.model.com_redhat_grid.Collector
        self.collector = RosemaryObjectParameter(app, "id", cls)
        self.add_parameter(self.collector)

        self.flash_chart = PoolSlotFlashVis(app, "chart", self.collector)
        self.flash_chart.fullpageable = False
        self.replace_child(self.flash_chart)

    def render_content(self, session):
        return self.flash_chart.render(session)
