import logging

from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *

from cumin.objectframe import *
from cumin.objectselector import *
from cumin.stat import *
from cumin.widgets import *
from cumin.OpenFlashChart import *

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.slot")

class SlotSelector(ObjectSelector):
    def __init__(self, app, name, pool):
        cls = app.model.com_redhat_grid.Slot

        super(SlotSelector, self).__init__(app, name, cls)

        self.pool = pool

        self.add_filter(self.pool, cls.Pool, cls.Pool)

        frame = "main.grid.pool.slot"
        col = ObjectLinkColumn(app, "name", cls.Name, cls._id, frame)
        self.add_column(col)

        self.add_attribute_column(cls.Activity)
        self.add_attribute_column(cls.State)
        self.add_attribute_column(cls.LoadAvg)

class SlotFrame(ObjectFrame):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Slot

        super(SlotFrame, self).__init__(app, name, cls)

class SlotDataSet(CuminSqlDataSet):
    def __init__(self, app):
        super(SlotDataSet, self).__init__(app)

        self.where_exprs.default = list()

    def render_sql_order_by(self, session):
        return "Order by \"System\",\"Name\""

class SlotView(CuminView):
    def __init__(self, app, name, slot):
        super(SlotView, self).__init__(app, name, slot)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)

        stats = SlotStats(app, "stats", slot)
        self.tabs.add_tab(stats)

        details = CuminDetails(app, "details", slot)
        self.tabs.add_tab(details)

class SlotLoadStatSet(StatSet):
    def __init__(self, app, name, object):
        super(SlotLoadStatSet, self).__init__(app, name, object)

        self.attrs = ("CondorLoadAvg", "LoadAvg"),

class SlotStats(Widget):
    def __init__(self, app, name, slot):
        super(SlotStats, self).__init__(app, name)

        stats = SlotLoadStatSet(app, "general", slot)
        self.add_child(stats)

        job_info = SlotJobInfo(app, "job_info", slot, "JobInfo")
        self.add_child(job_info)

        chart = self.LoadChart(app, "chart", slot)
        chart.stats = ("LoadAvg", "CondorLoadAvg")
        chart.duration.param.default = "3600"
        self.add_child(chart)

    def render_title(self, session):
        return "Statistics"

    class LoadChart(StatFlashChart):
        def render_title(self, session):
            return "Load"

class SlotJobInfo(PropertySet):
    def __init__(self, app, name, slot, category):
        super(SlotJobInfo, self).__init__(app, name)

        #self.defer_enabled = True
        self.update_enabled = True
        self.category = category
        self.object = slot

    def do_get_items(self, session):
        obj = self.object.get(session)
        cls = self.app.model.get_class_by_object(obj)

        return [(x.get_title(session), x.value(session, obj), x.escape)
                for x in cls.properties if x.category == self.category]

class SlotMapPage(Page):
    """ handles the slots.vis request """

    interiors = {"Idle": (.7,.7,.7),
               "Busy": (0.0, 0.4, 0.0),
               "Suspended": (1,0,0),
               "Vacating": (1,.73,.367),
               "Killing": (0,0,1),
               "Benchmarking": (1,.8,.8),
               "Retiring": (.8,.2,.8),
               "Unknown": (.8,.8,.8)}

    shapes = ["rectangle", "circle", "circle"]
    max_width = 400
    max_png_age = 30
    max_visible_slots = 200 # start drill-down mode after this number
    def __init__(self, app, name, param, desc):
        super(SlotMapPage, self).__init__(app, name)

        self.object_param = param
        self.object_description = desc

        self.zoom_level = IntegerParameter(app, "zl")
        self.zoom_level.default = 1
        self.add_parameter(self.zoom_level)

        self.activity = Parameter(app, "act")
        self.add_parameter(self.activity)

        self.state = Parameter(app, "state")
        self.add_parameter(self.state)

        self.json = Parameter(app, "json")
        self.add_parameter(self.json)

        self.expanded = Parameter(app, "expanded")
        self.expanded.default = ""
        self.expanded.all = "all"
        self.add_parameter(self.expanded)

        group = Parameter(app, "agroup")
        self.groups = ListParameter(app, "group", group)
        self.add_parameter(self.groups)

        self.cache = ImageCache()
        self.slots = SlotDataSet(app)

    def get_content_type(self, session):
        return self.json.get(session) and "text/plain" or "image/png"

    def get_cache_control(self, session):
        return "no-cache"

    def render_activity(self, session, activity):
        map = HeatMapChart()
        surface = map.plot_colored_rect(self.interiors[activity], 12, 12)
        writer = Writer()
        surface.write_to_png(writer)
        return writer.to_string()

    def render_state(self, session, state):
        map = HeatMapChart()
        surface = map.plot_state(state, 12, 12)
        writer = Writer()
        surface.write_to_png(writer)
        return writer.to_string()

    def do_render(self, session):
        activity = self.activity.get(session)
        if activity:
            return self.render_activity(session, activity)

        state = self.state.get(session)
        if state:
            return self.render_state(session, state)

        if self.json.get(session):
            return self.render_json(session)

        zl = self.zoom_level.get(session)

        # determine if cached copy is recent enough
        cached_png, args = self.get_cached(session, zl)
        if cached_png:
            self.set_cookie(session, args)
            return cached_png

        map = HeatMapChart(self.max_width, self.max_width)

        cursor = self.slots.execute(session)
        records = cursor.fetchall()

        if len(records) == 0:
            return

        columns = [x[0] for x in cursor.description]
        activity = columns.index("Activity")
        state = columns.index("State")

        interiors = self.interiors.copy()
        interiors[None] = interiors["Unknown"]
        slots = [(interiors[x[activity]], x[state]) for x in records]
        size = map.plot_slots(slots, zl)

        args = (size, map.width, map.height, len(slots), map.rows, map.cols)
        self.set_cookie(session, args)
        self.cache_it(session, zl, map, args)

        writer = Writer()
        map.write(writer)
        return writer.to_string()

    def get_info_array(self, records, array, attr):
        d = dict()
        for i in array:
            atr = records[i][attr]
            if not atr:
                atr = "Unknown %s" % attr
            if not atr in d:
                d[atr] = list()
            d[atr].append(i)
        return d

    def get_activityStates(self, records, plist):
        """ for each unique combination of activity:state, return a dictionary entry
            that counts the number of records with that combo """

        d = dict()
        for i in plist:
            activityState = "%s:%s" % (records[i]["Activity"], records[i]["State"])
            if not activityState in d:
                d[activityState] = 0
            d[activityState] += 1
        return d

    def render_json(self, session):
        cursor = self.slots.execute(session)
        records = cursor_to_rows(cursor)

        if len(records) == 0:
            return "[]"

        groups = self.groups.get(session)
        slot_count = len(records)

        root = Element()
        root.name = self.object_description
        root.value = self.object_param.get(session)._id
        root.slots = slot_count
        root.vis = self.json.get(session)
        root.activity_colors = self.interiors
        expanded = self.expanded.get(session)
        json = self.json.get(session)
        leaves = True
        (root.tree, root.back) = self.treeify(records, range(slot_count), groups, 0, 
                                 expanded, leaves, slot_count, json)

        return "[%s]" % root.create()

    def treeify(self, records, plist, groups, level, expanded, leaves, slot_count, json):
        level_list = list()
        # leaf
        if level == len(groups):
            if leaves:
                for i in sorted(plist, key=lambda x:"%s%s%s" % 
                                (records[x]["Activity"], records[x]["State"], records[x]["Name"])):
                    el = Element()
                    el.job_id = records[i]["JobId"] and records[i]["JobId"] or ""
                    el.activity = records[i]["Activity"] and records[i]["Activity"] or "Unknown"
                    el.state = records[i]["State"] and records[i]["State"] or "Unknown"
                    el.value = records[i]["Name"] and records[i]["Name"] or ""
                    el.load_avg = records[i]["LoadAvg"] and round(records[i]["LoadAvg"], 2) or 0
                    el.name = "slot"
                    el.slot_id = records[i]["_id"]
                    level_list.append(el)
                return (level_list, True)
            else:
                if not expanded:
                    # display summary info for all the slots under this grouping
                    activityStates = self.get_activityStates(records, plist)
                    for activityState in sorted(activityStates):
                        el = Element()
                        el.name = "slot_info"
                        el.a_s = activityState
                        el.count = activityStates[activityState]
                        el.value = "%s.%s" % (records[plist[0]][groups[level-1]], activityState) # uid
                        level_list.append(el)
                return (level_list, False)

        # not a leaf
        group = groups[level]
        level_dict = self.get_info_array(records, plist, group)
        first = True
        for key in sorted(level_dict):
            el = Element()
            el.name = group
            el.value = key
            el.slots = len(level_dict[key])
            el.level = level
            if level < len(groups):
                if json == "slots":
                    if expanded == key:
                        leaves = True
                    elif expanded == "None" and key is None:
                        leaves = True
                    else:
                        leaves = False
                (el.tree, el.back) = self.treeify(records, level_dict[key], groups, level + 1, 
                                       expanded, leaves, slot_count, json)
                
            if len(el.tree):
                level_list.append(el)

        return (level_list, False)

    def get_cached(self, session, zl):
        filename = self.gen_filename(session)
        return self.cache.find_recent(filename, self.max_png_age)

    def cache_it(self, session, zl, map, args):
        filename = self.gen_filename(session)
        writer = self.cache.create_cache_file(filename, args)
        map.write(writer)

    def gen_filename(self, session):
        return session.marshal()

    def set_cookie(self, session, args):
        cookie = "|".join((str(x) for x in args))
        session.set_cookie("slot_info", cookie)

class SlotMap(Widget):
    def __init__(self, app, name):
        super(SlotMap, self).__init__(app, name)

        self.slot_info = self.SlotInfo(app, "slot_info")
        self.add_child(self.slot_info)

        self.slot_legend = self.SlotLegend(app, "slot_legend")
        self.add_child(self.slot_legend)

        self.slot_clip_size = 400

        self.slots = SlotDataSet(app)

    def render_image_href(self, session):
        raise Exception("Not implemented")

    def render_job_index_param(self, session):
        return "xargs"

    def render_fullpageable(self, session):
        return "fullpageable"

    def get_scheduler_select(self, session):
        raise Exception("Must implement get_scheduler_select")

    def render_slot_url(self, session):
        slot = Identifiable("XXX")
        try:
            return self.page.main.grid.pool.slot.get_href(session, slot)
        except:
            pass

    def render_slot_clip_size(self, session):
        return self.slot_clip_size

    class SlotLegend(Widget):
        def __init__(self, app, name):
            super(SlotMap.SlotLegend, self).__init__(app, name)

            self.states = SlotStates(app, "slot_states")
            self.add_child(self.states)

            self.activities = SlotActivities(app, "slot_activities")
            self.add_child(self.activities)

    class SlotInfo(ItemSet):
        display_names = {"JobId": ("Job ID", "", ""),
                         "System": ("System", "", ""), 
                         "Machine": ("Machine", "", ""),
                         "State": ("State", "", ""),
                         "Activity": ("Activity", "", ""),
                         "Name": ("Name", "", ""),
                         "_id": ("Slot_ID", "slotInfo_id", "hidden_row")}

        def __init__(self, app, name):
            super(SlotMap.SlotInfo, self).__init__(app, name)

            self.index = IntegerParameter(app, "i")
            self.add_parameter(self.index)

            self.slot = Attribute(app, "slot")
            self.add_attribute(self.slot)

            self.info_div_tmpl = WidgetTemplate(self, "bg_html")

        def do_render(self, session):
            index = self.index.get(session)

            if index is not None:
                self.parent.slots.limit.set(session, 1)
                self.parent.slots.offset.set(session, index)

                cursor = self.parent.slots.execute(session)

                data = dict()
                names = [x[0] for x in cursor.description]
                values = cursor.fetchone()

                for name, value in zip(names, values):
                    data[name] = value

                self.slot.set(session, data)

                writer = Writer()
                self.info_div_tmpl.render(writer, session)
                return writer.to_string()
            else:
                return super(SlotMap.SlotInfo, self).do_render(session)

        def do_get_items(self, session):
            slot = self.slot.get(session)

            return ((self.display_names[x], slot[x])
                    for x in slot if x in self.display_names)

        def render_slot_info_url(self, session):
            return self.page.get_update_url(session, (self,))

        def render_slot_info_index(self, session):
            return self.index.path

        def render_item_value(self, session, item):
            return item[1]

        def render_item_title(self, session, item):
            return item[0][0]

        def render_item_id(self, session, item):
            return item[0][1] and "id='%s'" % item[0][1] or ""

        def render_item_row_class(self, session, item):
            return item[0][2] and "class='%s'" % item[0][2] or ""

class SlotStates(ItemSet):
    states = ("Unclaimed", "Claimed", "Owner", "Matched", "Preempting")

    def do_get_items(self, session):
        return self.states

    def render_item_size(self, session, activity):
        return 12

    def render_item_title(self, session, state):
        return state

    def render_item_href(self, session, state):
        params = list()
        params.append("state=%s" % state)

        return "poolslots.vis?" + ";".join(params)

class SlotActivities(ItemSet):
    activities = (("Idle", "clear"),
                  ("Busy", "green"),
                  ("Suspended", "red"),
                  ("Vacating", "orange"),
                  ("Killing", "blue"),
                  ("Benchmarking", "yellow"),
                  ("Retiring", "purple"),
                  ("Unknown", "grey"))

    def do_get_items(self, session):
        return self.activities

    def render_item_size(self, session, activity):
        return 12

    def render_item_title(self, session, activity):
        return activity[0]

    def render_item_href(self, session, activity):
        params = list()
        params.append("act=%s" % activity[0])

        return "poolslots.vis?" + ";".join(params)

