import logging
import os
import sys

from mint import *
from parsley.loggingex import *
from stat import StatChartPage, StatStackedPage, \
    StatFlashPage, FlashFullPage
from wooly import Application, Session, Page
from wooly.pages import ResourcePage
from wooly.parameters import IntegerParameter

from admin import *
from database import *
from model import *
from objectselector import *
from objecttask import *
from server import *
from session import *
from sqladapter import *
from user import *
from util import *
from widgets import *

from wooly import Session

strings = StringCatalog(__file__)
log = logging.getLogger("cumin")

class Cumin(Application):
    def __init__(self, home, broker_uris, database_dsn,
                 host="localhost", port=45672):
        super(Cumin, self).__init__()

        self.home = home

        model_dir = os.path.join(self.home, "model")

        self.model = CuminModel(self, model_dir)
        self.session = CuminSession(self, broker_uris)
        self.database = CuminDatabase(self, database_dsn)
        self.server = CuminServer(self, host, port)
        self.admin = CuminAdmin(self)

        self.add_resource_dir(os.path.join(self.home, "resources-wooly"))
        self.add_resource_dir(os.path.join(self.home, "resources"))

        self.modules = list()
        self.modules_by_name = dict()

        self.tasks = list()

        self.user = None
        self.operator_email = None
        self.update_interval = 10

        # self.model.sql_logging_enabled = True

    def check(self):
        log.info("Checking %s", self)

        if not os.path.isdir(self.home):
            msg = "Cumin home '%s' not found or not a directory"
            raise Exception(msg % self.home)

        self.model.check()

    def init(self):
        log.info("Initializing %s", self)

        self.model.init()
        self.session.init()
        self.database.init()
        self.server.init()

        self.add_pages()

        import account
        import messaging
        import grid
        import inventory
        import usergrid

        account.Module(self, "account")
        messaging.Module(self, "messaging")
        grid.Module(self, "grid")
        inventory.Module(self, "inventory")
        usergrid.Module(self, "usergrid")

        for module in self.modules:
            module.init()

        for task in self.tasks:
            task.init()

        super(Cumin, self).init()

    def add_pages(self):
        self.main_page = MainPage(self, "index.html")

        self.add_page(self.main_page)
        self.set_default_page(self.main_page)

        self.form_page = CuminFormPage(self, "form.html")
        self.add_page(self.form_page)

        self.add_page(CallPage(self, "call.xml"))
        self.add_page(StatChartPage(self, "stats.png"))
        self.add_page(StatStackedPage(self, "stacked.png"))
        self.add_page(StatFlashPage(self, "chart.json"))
        self.add_page(FlashFullPage(self, "flashpage.html"))

        self.resource_page.protected = False

    def start(self):
        log.info("Starting %s", self)

        self.session.start()
        self.server.start()

    def stop(self):
        log.info("Stopping %s", self)

        self.server.stop()
        self.session.stop()

class CuminModule(object):
    def __init__(self, app, name):
        self.app = app
        self.name = name

        self.tasks = list()

        assert not hasattr(app, self.name), self.name

        self.app.modules.append(self)
        setattr(self.app, self.name, self)

    def init(self):
        for task in self.tasks:
            task.init()

    def init_test(self, test):
        pass

class MainPage(CuminPage, ModeSet):
    def __init__(self, app, name):
        super(MainPage, self).__init__(app, name)

        self.main = MainView(app, "main")
        self.add_mode(self.main)
        self.set_default_frame(self.main)

        self.page_html_class = "Cumin"

    def render_title(self, session):
        return self.get_title(session)

    def get_title(self, session):
        return "Administrator"

class MainView(CuminMainView):
    def __init__(self, app, name):
        super(MainView, self).__init__(app, name)

        self.overview = OverviewFrame(app, "overview")
        self.add_tab(self.overview)

# XXX Add qmf tab

class OverviewFrame(CuminFrame):
    def __init__(self, app, name):
        super(OverviewFrame, self).__init__(app, name)

        self.view = OverviewView(app, "view")
        self.add_mode(self.view)

        self.notice = ConfigurationNotice(app, "notice")
        self.add_mode(self.notice)

    def do_process(self, session):
        super(OverviewFrame, self).do_process(session)

        count = len(self.app.session.qmf_brokers)

        if count == 0:
            self.mode.set(session, self.notice)

    def render_title(self, session):
        return "Overview"

class OverviewView(Widget):
    def __init__(self, app, name):
        super(OverviewView, self).__init__(app, name)

        queues = TopQueueTable(app, "queues")
        self.add_child(queues)

        systems = TopSystemTable(app, "systems")
        self.add_child(systems)

        submissions = TopSubmissionTable(app, "submissions")
        self.add_child(submissions)

        # XXX

        #queues = messaging.queue.TopQueueSet(app, "queues")
        #self.add_child(queues)

        #submissions = grid.submission.TopSubmissionSet(app, "submissions")
        #self.add_child(submissions)

        #systems = inventory.system.TopSystemSet(app, "systems")
        #self.add_child(systems)

    class Heading(CuminHeading):
        def render_title(self, session):
            return "Overview"

        def render_icon_href(self, session):
            return "resource?name=action-36.png"

class ConfigurationNotice(Widget):
    pass

class TopQueueAdapter(SqlAdapter):
    def __init__(self, app, cls):
        super(TopQueueAdapter, self).__init__(app, cls.sql_table)
        self.cls = cls

    def init(self):
        super(TopQueueAdapter, self).init()

        name_col = self.table._columns_by_name["name"]
        avg_over_last_60_seconds_col = """((sum("Queue"."msgTotalEnqueues") - 
            sum(s."msgTotalEnqueues")) / (count(1)-1)) / 300 as avg_60"""
        queue_id_col = self.table._columns_by_name["_id"]
        vhostRef_col = self.table._columns_by_name["_vhostRef_id"]
        vhost_table = self.app.model.org_apache_qpid_broker.Vhost.sql_table
        vhost_id_col = vhost_table._columns_by_name["_id"]
        vhost_brokerRef_col = vhost_table._columns_by_name["_brokerRef_id"]

        self.columns.append(name_col)
        self.columns.append(avg_over_last_60_seconds_col)
        self.columns.append(vhost_brokerRef_col)
        self.columns.append(queue_id_col)

        sub_query = "(%s) as s" % self.get_sub_query_text()
        SqlInnerJoin(self.query, sub_query,
                     queue_id_col, "s._parent_id")

        SqlInnerJoin(self.query, vhost_table,
             vhostRef_col, vhost_id_col)

    def get_sub_query_text(self):
        samples_table = self.cls.sql_samples_table
        subquery = SqlQuery(samples_table)
        parent_col = samples_table._columns_by_name["_parent_id"]
        updated_col = samples_table._columns_by_name["_qmf_update_time"]
        enqueues_col = samples_table._columns_by_name["msgTotalEnqueues"]

        when = "now() - interval '600 seconds'"
        filter = SqlComparisonFilter(updated_col, when, ">=")
        subquery.add_filter(filter)

        columns = list()
        columns.append(parent_col)
        columns.append(enqueues_col)

        return subquery.emit(columns)

    def get_data(self, values, options):
        options.sort_column = "avg_60"
        options.sort_ascending = False

        options.group_column = "%s, %s, %s" % ("\"Queue\".name", "\"Queue\"._id", "\"Vhost\".\"_brokerRef_id\"")
        having = SqlComparisonFilter("count(1)", "1", ">")
        options.group_having.append(having)

        try:
            data = super(TopQueueAdapter, self).get_data(values, options)
        except:
            data = []
        return data

    def get_sql_options(self, options):
        return options

class TopQueueTable(TopTable):
    def __init__(self, app, name):
        cls = app.model.org_apache_qpid_broker.Queue
        super(TopQueueTable, self).__init__(app, name, cls)

        frame = "main.messaging.broker.queue"
        col = self.NameColumn(app, "name", cls.name, cls._id, frame)
        self.add_column(col)

        col = self.add_attribute_column(cls.msgDepth)

        self.sort.default = col.name

    def init(self):
        super(TopQueueTable, self).init()

        self.adapter.vhost_id_field = ObjectSqlField \
            (self.adapter, self.cls.vhostRef)

        filter = SqlComparisonFilter(self.cls.msgDepth.sql_column,
                                     "null",
                                     "is not")
        self.adapter.query.add_filter(filter)

    def render_rows(self, session):
        data = self.data.get(session)

        writer = Writer()

        for record in data:
            # if there is no broker id, don't render the row
            if record[self.adapter.vhost_id_field.index]:
                writer.write(self.row.render(session, record))

        return writer.to_string()

    class NameColumn(ObjectLinkColumn):
        def render_cell_href(self, session, record):
            branch = session.branch()

            frame = self.page.main.messaging.broker
            adapter = self.table.adapter

            frame.id.set(branch, record[adapter.vhost_id_field.index])
            frame.queue.id.set(branch, record[adapter.id_field.index])
            frame.queue.view.show(branch)

            return branch.marshal()

class TopSystemTable(TopTable):
    def __init__(self, app, name):
        cls = app.model.com_redhat_sesame.Sysimage

        super(TopSystemTable, self).__init__(app, name, cls)

        frame = "main.inventory.system"
        col = ObjectLinkColumn(app, "name", cls.nodeName, cls._id, frame)
        self.add_column(col)

        col = self.add_attribute_column(cls.loadAverage1Min)

        self.sort.default = col.name

    def init(self):
        super(TopSystemTable, self).init()

        filter = SqlComparisonFilter(self.cls.loadAverage1Min.sql_column,
                                     "null",
                                     "is not")
        self.adapter.query.add_filter(filter)

class TopSubmissionTable(TopTable):
    def __init__(self, app, name):
        cls = app.model.com_redhat_grid.Submission

        super(TopSubmissionTable, self).__init__(app, name, cls)

        col = self.NameColumn(app, cls.Name.name, cls.Name, cls._id, None)
        self.add_column(col)

        col = self.DurationColumn(app, cls._qmf_create_time.name,
                                  cls._qmf_create_time)
        self.add_column(col)

        self.sort.default = col.name

    def init(self):
        super(TopSubmissionTable, self).init()

        table = self.cls.sql_table

        filter = SqlComparisonFilter(table.Running, "0", ">")
        self.adapter.query.add_filter(filter)

    def render_rows(self, session):
        data = self.data.get(session)

        writer = Writer()

        for record in data:
            # if we can't get the collector, don't render the row
            submission_id = record[0]
            collector = self.get_collector(session, submission_id)
            if collector:
                writer.write(self.row.render(session, record))

        return writer.to_string()

    def get_collector(self, session, submission_id):
        try:
            cls = self.cls
            submission = cls.get_object_by_id(session.cursor, submission_id)

            cls = self.app.model.com_redhat_grid.JobServer
            job_server = cls.get_object_by_id(session.cursor, submission._jobserverRef_id)

            cls = self.app.model.com_redhat_grid.Collector
            return cls.get_object(session.cursor, Pool=job_server.Pool)
        except Exception, e:
            log.exception(e)

    class NameColumn(ObjectLinkColumn):
        def render_cell_href(self, session, record):
            submission_id = record[0]
            collector = self.parent.get_collector(session, submission_id)
            if collector:
                branch = session.branch()
                self.page.main.grid.pool.id.set(session, collector._id)
                self.page.main.grid.pool.submission.id.set(session, submission_id)
                self.page.main.grid.pool.submission.view.show(session)
                return branch.marshal()

    class DurationColumn(TopTableColumn):
        def render_header_content(self, session):
            return "Duration"

        def render_cell_content(self, session, record):
            created = self.field.get_content(session, record)
            return fmt_duration(time.time() - secs(created))

        def render_text_align(self, session):
            return "right"
