from decimal import *
from mint import *
from rosemary.model import *
from struct import unpack, calcsize
from threading import Thread, Lock
from types import *
from wooly import *
from wooly.parameters import *
from wooly.widgets import *
from wooly.datatable import *

from formats import *
from parameters import *
from sqladapter import *
from util import *

import wooly

log = logging.getLogger("cumin.model")

class CuminModel(RosemaryModel):
    def __init__(self, app, model_dir):
        super(CuminModel, self).__init__()

        self.app = app
        self.model_dir = model_dir

        self.tasks = list()
        self.task_invocations = list()

        self.limits_by_negotiator = dict()
        self.job_summaries_by_submission = dict()
        self.group_names_by_negotiator = dict()
        self.group_config_values_by_negotiator = dict()

        self.lock = Lock()

        self.job_meta_data = JobMetaData("job")

    def check(self):
        log.info("Checking %s", self)

        assert os.path.isdir(self.model_dir)

        log.debug("Model dir exists at '%s'", self.model_dir)

    def init(self):
        log.info("Initializing %s", self)

        self.load_model_dir(self.model_dir)

        super(CuminModel, self).init()

    def get_ad_groups(self):
        return AdProperty.get_ad_groups()

    def get_class_by_rosemary_object(self, rosemary_object):
        for cls in self.classes:
            if cls.cumin_name.lower() == rosemary_object._class._name.lower():
                return cls

    def get_class_by_object(self, mint_object):
        for cls in self.classes:
            if cls.mint_class is mint_object.__class__:
                return cls

    def show_main(self, session):
        return self.app.main_page.main.show(session)

    def get_session_by_object(self, obj):
        agent = self.app.session.get_agent(obj._qmf_agent_id)
        broker = agent.getBroker()
        session = broker.getAmqpSession()

        return session

    def get_negotiator_group_names(self, negotiator):
        assert negotiator

        self.lock.acquire()

        try:
            try:
                store = self.group_names_by_negotiator[negotiator._qmf_agent_id]
            except KeyError:
                store = NegotiatorGroupNamesStore(self, negotiator)
                store.start_updates()

                self.group_names_by_negotiator[negotiator._qmf_agent_id] = store
                #sleep(1)

            return store
        finally:
            self.lock.release()

    def get_negotiator_config_values(self, negotiator, needed_groups, config):
        assert negotiator

        self.lock.acquire()

        try:
            try:
                store = self.group_config_values_by_negotiator[negotiator._qmf_agent_id]
                added = 0
                for group in needed_groups:
                    added += store.add_group_config(group, config)
                if added > 0:
                    store.update(None)
                    sleep(1)
            except KeyError:
                store = NegotiatorGroupConfigValuesStore(self, negotiator, needed_groups, config)
                store.start_updates()

                self.group_config_values_by_negotiator[negotiator._qmf_agent_id] = store

                sleep(1)

            return store
        finally:
            self.lock.release()

    def update_negotiator_config_value(self, negotiator):
        assert negotiator

        self.lock.acquire()

        try:
            store = self.group_config_values_by_negotiator[negotiator._qmf_agent_id]
            store.update(None)
            sleep(1)
        finally:
            self.lock.release()

    def get_negotiator_limits(self, negotiator):
        assert negotiator

        self.lock.acquire()

        try:
            try:
                store = self.limits_by_negotiator[negotiator._qmf_agent_id]
            except KeyError:
                store = NegotiatorLimitStore(self, negotiator)
                store.start_updates()

                self.limits_by_negotiator[negotiator._qmf_agent_id] = store

                sleep(1)

            return store.data
        finally:
            self.lock.release()

    def get_submission_job_summaries(self, submission):
        assert submission

        self.lock.acquire()

        try:
            try:
                store = self.job_summaries_by_submission[submission._id]
            except KeyError:
                store = SubmissionJobSummaryStore(self, submission)
                store.start_updates()

                self.job_summaries_by_submission[submission._id] = store

                for i in range(5):
                    if store.data or store.status:
                        break

                    sleep(1)

            return store
        finally:
            self.lock.release()

class CuminProperty(object):
    def __init__(self, cls, name):
        self.model = cls.model
        self.cumin_class = cls

        self.name = name
        self.title = None
        self.category = "general"
        self.summary = False
        self.escape = True
        self.prefix = None

        self.cumin_class.add_property(self)

    def init(self):
        pass

    def get_title(self, session):
        if self.title:
            return self.title
        else:
            return self.name

    def value(self, session, object):
        value = getattr(object, self.name, None)

        if isinstance(value, datetime):
            value = fmt_datetime(value)

        if isinstance(value, dict):
            value = fmt_dict(value, self.prefix)

        return value

class AdProperty(object):
    groups = ["Main", "Command Info", "Job Status Info", "Condor Info", "Dates", "Other"]

    def __init__(self, cls, name):
        # don't call super since we don't want to call add_property
        self.cumin_class = cls

        self.name = name
        self.title = None
        self.category = "general"
        self.summary = False
        self.escape = True
        self.prefix = None

        self.description = None
        self.example = None
        self.group = "Main"
        self.renderer = None
        self.writable = True

        self.cumin_class.add_ad_property(self)

    def get_title(self, session):
        if self.title:
            return self.title
        else:
            return self.name

    def value(self, session, object):
        value = getattr(object, self.name, None)

        if isinstance(value, datetime):
            value = fmt_datetime(value)

        if isinstance(value, dict):
            value = fmt_dict(value, self.prefix)

        return value

    @classmethod
    def get_ad_groups(cls):
        return cls.groups

class DictAdProperty(AdProperty):
    def __init__(self, cls, name):
        super(DictAdProperty, self).__init__(cls, name)

        self.renderer = self.render_dict
        self.group = "Other"

    def render_dict(self, session, value):
        return fmt_dict(value, self.prefix)

class DateAdProperty(AdProperty):
    def __init__(self, cls, name):
        super(DateAdProperty, self).__init__(cls, name)

        self.renderer = self.render_datetime
        self.group = "Other"

    def render_datetime(self, session, value):
        value = datetime.fromtimestamp(int(value))
        return fmt_datetime(value)

class CuminAction(object):
    def __init__(self, cls, name):
        self.model = cls.model
        self.mint = self.model.mint
        self.cumin_class = cls

        self.name = name
        self.title = None
        self.summary = False
        self.navigable = True
        self.aggregate = False

        self.cumin_class.add_action(self)

    def init(self):
        pass

    def show(self, session, object):
        raise Exception("Not implemented")

    # XXX we could even cache this if it gets called a lot
    def get_href(self, session, object):
        branch = session.branch()
        self.show(branch, object)
        return branch.marshal()

    def get_title(self, session):
        if self.title:
            return self.title
        else:
            return self.name

    def get_summary(self, session, object):
        pred0 = self.get_title(session)
        pred1 = self.cumin_class.get_object_title(session, object)

        return "%s %s" % (pred0, pred1)

    def get_enabled(self, session, object):
        return True

    def get_verb(self, session):
        return None

    def get_description(self, session, object):
        verb  = self.get_verb(session)

        if not verb:
            verb = self.get_title(session)

        cls = self.cumin_class

        if cls and object:
            obj = None

            if type(object) is list:
                if len(object) == 1:
                    obj = object[0]
            else:
                obj = object

            if obj is None:
                sobject = "%i %s" % \
                    (len(object), cls.get_plural_title(session))
            else:
                sobject = cls.get_object_title(session, obj)
        elif cls:
            sobject = cls.get_title(session)
        else:
            # XXX lame
            sobject = str(object)

        return "%s %s" % (verb, sobject)

    def invoke(self, object, args={}):
        invoc = self.begin(object)

        def completion(status, args=None):
            invoc.status = status
            invoc.args = args
            #invoc.prt()

        try:
            return self.do_invoke(object, args, completion)
        except Exception, e:
            invoc.status = "failed"
            invoc.exception = e

            log.exception(e)

        #invoc.prt()

        return invoc

    def do_invoke(self, object, args, completion):
        raise Exception("Not implemented")

    def begin(self, object):
        invoc = CuminActionInvocation(self, object)
        self.model.invocations.add(invoc)
        return invoc

class CuminActionInvocation(object):
    def __init__(self, action, object):
        self.action = action
        self.object = object
        self.when = datetime.now()
        self.status = "pending"
        self.args = None
        self.exception = None

    def get_description(self, session):
        return self.action.get_description(session, self.object)

    def prt(self):
        print "action", self.action.name, self.object, self.when, \
            self.status, self.args, self.exception

class CuminSetAction(CuminAction):
    def __init__(self, cls, name):
        super(CuminSetAction, self).__init__(cls, name)

        self.aggregate = True

    def get_summary(self, session, object):
        pred0 = self.get_title(session)
        pred1 = self.cumin_class.get_plural_title(session)

        return "%s %s" % (pred0, pred1)

class CuminStat(object):
    def __init__(self, cls, name):
        self.model = cls.model
        self.cumin_class = cls

        self.name = name
        self.unit = None
        self.title = None
        self.category = "general"
        self.summary = False
        self.format = None

        self.link_cb = None
        self.highlow = False

        self.cumin_class.add_stat(self)

    def init(self):
        pass

    def get_connection(self):
        return self.app.database.get_connection()

    def get_db_name(self):
        name = self.name
        nname = list()
        for c in name:
            if c.isupper():
                if len(nname):
                    nname.append("_")
                c = c.lower()
            nname.append(c)
        return "".join(nname)

    def value_html(self, object):
        text = self.value_text(object)

        if text:
            html = text
        else:
            html = fmt_none_brief()

        return html

    def value_text(self, object):
        value = self.value(object)

        if value is None:
            text = ""
        elif self.format:
            text = self.format % value
        elif type(value) in (int, long, Decimal):
            if value == 0:
                text = "0"
            else:
                text = "%i" % value
        else:
            text = "%r" % value

        return text

    def value(self, object):
        try:
            return getattr(object, self.name)
        except AttributeError:
            pass

    def rate_html(self, object):
        text = self.rate_text(object)

        if text:
            html = text
        else:
            html = fmt_none_brief()

        return html

    def rate_text(self, object):
        rate = self.rate(object)

        if rate is None:
            text = ""
        elif rate == 0:
            text = "0"
        else:
            text = "%0.2f" % rate

        return text

    def rate(self, object):
        threshold = datetime.now() - timedelta(minutes=10)

        try:
            if object.statsCurr and object.statsCurr.qmfUpdateTime > threshold:
                curr = getattr(object.statsCurr, self.name)
                ctime = object.statsCurr.qmfUpdateTime
                csecs = mktime(ctime.timetuple())

                if object.statsPrev:
                    prev = getattr(object.statsPrev, self.name)
                    ptime = object.statsPrev.qmfUpdateTime
                    psecs = mktime(ptime.timetuple())

                    if curr is not None and prev is not None:
                        return calc_rate(curr, prev, csecs, psecs)
        except AttributeError:
            pass

    def write_xml(self, writer, object):
        value = self.value_text(object)
        rate = self.rate_text(object)

        writer.write("<stat name=\"%s\" value=\"%s\" rate=\"%s\"/>" \
                     % (self.name, value, rate))

class SamplesSqlAdapter(SqlAdapter):
    qmf_update_col = '_qmf_update_time'

    def __init__(self, app, table, agent_id, object_id):
        super(SamplesSqlAdapter, self).__init__(app, table)

        assert ";" not in agent_id
        assert ";" not in object_id

        col = self.table._qmf_agent_id
        filter = SqlComparisonFilter(col, "'%s'" % agent_id)
        self.query.add_filter(filter)

        col = self.table._qmf_object_id
        filter = SqlComparisonFilter(col, "'%s'" % object_id)
        self.query.add_filter(filter)

    def get_sql_options(self, options):
        return options

    def avg_samples(self, stat, secs, interval, secs2):
        stat_col = self.table._columns_by_name[stat.name]
        updated_col = self.table._columns_by_name[self.qmf_update_col]

        org_filters = list(self.query.filters)
        max_col = "max(%s) as interval_end" % updated_col.identifier
        value_col = "cast(avg(%s) as integer) as value" % stat_col.identifier
        dev_col = "stddev(%s) as dev" % stat_col.identifier

        columns = list()
        columns.append(max_col)
        columns.append(value_col)
        columns.append(dev_col)
        self.columns = columns

        when = "now() - interval '%i seconds'" % int(secs + secs2)
        filter = SqlComparisonFilter(updated_col, when, ">=")
        self.query.add_filter(filter)
        when2 = ""
        filter2 = None
        if secs2:
            when2 = "now() - interval '%i seconds'" % int(secs2)
            filter2 = SqlComparisonFilter(updated_col, when2, "<=")
            self.query.add_filter(filter2)

        options = SqlQueryOptions()
        options.sort_column = "interval_end"
        options.sort_ascending = False
        options.group_column = "floor(extract(epoch from %s) / %i)" % (updated_col.identifier, interval)

        samples = self.get_data({}, options)

        # reset the filters in case we need to re-run the query in this same request
        del self.query.filters[:]
        self.query.filters = org_filters

        return samples

    def samples(self, stat, secs, interval, method, secs2=0, delta=False):
        if method == "avg":
            return self.avg_samples(stat, secs, interval, secs2)

        org_filters = list(self.query.filters)
        stat_col = self.table._columns_by_name[stat.name]
        updated_col = self.table._columns_by_name[self.qmf_update_col]

        columns = list()
        columns.append(updated_col.identifier)
        columns.append(stat_col.identifier)
        self.columns = columns

        if not delta:
            when = "now() - interval '%i seconds'" % int(secs + secs2)
            filter = SqlComparisonFilter(updated_col, when, ">=")
            self.query.add_filter(filter)
        when2 = ""
        if secs2:
            when2 = "now() - interval '%i seconds'" % int(secs2)
            filter = SqlComparisonFilter(updated_col, when2, "<=")
            self.query.add_filter(filter)


        options = SqlQueryOptions()
        options.sort_column = updated_col
        options.sort_ascending = False
        if delta:
            options.limit = 2

        samples = self.get_data({}, options)

        # restore the filters
        del self.query.filters[:]
        self.query.filters = org_filters

        return samples

    def recent(self):
        updated_col = self.table._columns_by_name[self.qmf_update_col]

        columns = list()
        columns.append(updated_col.identifier)
        self.columns = columns

        options = SqlQueryOptions()
        options.sort_column = updated_col
        options.sort_ascending = False

        samples = self.get_data({}, options)
        return len(samples) and samples[0][0] or None

class CuminStatistic(object):
    formatters = {
        NoneType: lambda x: fmt_none(),
        float: lambda x: "%0.2f" % x,
        Decimal: lambda x: str(x),
        int: lambda x: "%i" % x
        }

    def __init__(self, name):
        self.name = name

    def get_title(self, session):
        return self.name

    def format_value(self, session, value):
        return CuminStatistic.fmt_value(value)

    @classmethod
    def fmt_value(cls, value):
        try:
            return cls.formatters[type(value)](value)
        except KeyError:
            return value

class MetaData(object):
    def __init__(self, name):

        self.name = name
        self.ad_properties = list()
        self.ad_properties_by_name = dict()

    def init(self):
        for ad_prop in self.ad_properties:
            ad_prop.init()

    def add_ad_property(self, prop):
        self.ad_properties.append(prop)
        self.ad_properties_by_name[prop.name] = prop

class CuminClass(object):
    def __init__(self, model, name, mint_class):
        self.model = model
        self.cumin_name = name
        self.mint_class = mint_class

        self.properties = list()
        self.ad_properties = list()
        self.ad_properties_by_name = dict()
        self.stats = list()
        self.actions = list()

        self.tasks = list()
        self.tasks_by_class = dict()

        self.frame = None

        self.model.add_class(self)

    def add_property(self, prop):
        self.properties.append(prop)
        setattr(self, prop.name, prop)

    def add_ad_property(self, prop):
        self.ad_properties.append(prop)
        self.ad_properties_by_name[prop.name] = prop

    def add_stat(self, stat):
        self.stats.append(stat)
        setattr(self, stat.name, stat)

    def add_action(self, action):
        self.actions.append(action)
        setattr(self, action.name, action)

    def add_task(self, task):
        self.tasks.append(task)
        self.tasks_by_class[task.__class__] = task

    def init(self):
        log.debug("Initializing class %s", self)

        for prop in self.properties:
            prop.init()

        for ad_prop in self.ad_properties:
            ad_prop.init()

        for stat in self.stats:
            stat.init()

        for action in self.actions:
            action.init()

        for task in self.tasks:
            log.debug("Initializing task %s", task)
            task.init()

    def get_title(self, session):
        return "Object"

    def get_plural_title(self, session):
        return self.get_title(session) + "s"

    def get_icon_href(self, session):
        return "resource?name=action-36.png"

    def get_object_href(self, session, object):
        branch = session.branch()
        self.show_object(branch, object)
        return branch.marshal()

    def get_object_href_by_id(self, session, id):
        return self.get_object_href(session, Identifiable(id))

    def show_object(self, session, object):
        return self.frame.show_object(session, object)

    def get_object_title(self, session, object):
        title = self.get_title(session)
        name = self.get_object_name(object)
        return "%s '%s'" % (title, name)

    def get_object_name(self, object):
        return object.name

    def write_event_xml(self, writer, object):
        writer.write("<events errors=\"%i\" warnings=\"%i\"/>" % (0, 0))

    def write_stat_xml(self, session, writer, object):
        for stat in self.stats:
            stat.write_xml(writer, object)

    def write_xml(self, session, writer, object):
        object.sync()

        writer.write("<%s id=\"%s\" name=\"%s\">" % \
                         (self.cumin_name, str(object.id),
                          self.get_object_name(object)))

        self.write_event_xml(writer, object)
        self.write_stat_xml(session, writer, object)

        writer.write("</%s>" % self.cumin_name)

    def __str__(self):
        return "%s.%s" % (self.__module__, self.__class__.__name__)

class RemoteClass(CuminClass):
    def __init__(self, model, name, mint_class, mint_stats_class):
        super(RemoteClass, self).__init__(model, name, mint_class)

        self.mint_stats_class = mint_stats_class

        prop = CuminProperty(self, "qmfObjectId")
        prop.title = "Management ID"

        prop = CuminProperty(self, "qmfCreateTime")
        prop.title = "Creation Time"

        prop = CuminProperty(self, "qmfDeleteTime")
        prop.title = "Deletion Time"

        prop = CuminProperty(self, "qmfUpdateTime")
        prop.title = "Last Updated"
        prop.summary = True

class CuminGrid(RemoteClass):
    def __init__(self, model):
        super(CuminGrid, self).__init__(model, "grid", Grid, GridStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        prop = CuminProperty(self, "Pool")
        prop.title = "Pool"

        prop = CuminProperty(self, "JobLimit")
        prop.title = "Job Limit"

        prop = CuminProperty(self, "SubmitLimit")
        prop.title = "Submit Limit"

        prop = CuminProperty(self, "GridResourceUnavailableTime")
        prop.title = "Grid Resource Unavailable Time"

        stat = CuminStat(self, "NumJobs")
        stat.title = "Num Jobs"
        stat.category = "general"

        stat = CuminStat(self, "SubmitsInProgress")
        stat.title = "Submits In Progress"
        stat.category = "general"

        stat = CuminStat(self, "SubmitsQueued")
        stat.title = "Submits Queued"
        stat.category = "general"

        stat = CuminStat(self, "SubmitsAllowed")
        stat.title = "Submits Allowed"
        stat.category = "general"

        stat = CuminStat(self, "SubmitsWanted")
        stat.title = "Submits Wanted"
        stat.category = "general"

        stat = CuminStat(self, "RunningJobs")
        stat.title = "Running Jobs"
        stat.category = "general"

        stat = CuminStat(self, "IdleJobs")
        stat.title = "Idle Jobs"
        stat.category = "general"

    def init(self):
        super(CuminGrid, self).init()

    def get_title(self, session):
        return "Grid"

    def get_icon_href(self, session):
        return "resource?name=pool-36.png"

# XXX "do_" on this doesn't make sense
def do_bind(session, queue_name, binding_info):
        for exchange in binding_info:
            if "key" in binding_info[exchange]:
                binding_key = binding_info[exchange]["key"]
            else:
                binding_key = None

            session.exchange_bind(queue=queue_name,
                exchange=binding_info[exchange]["name"],
                binding_key=binding_key,
                arguments=binding_info[exchange]["arguments"])

class CuminQueue(RemoteClass):
    def __init__(self, model):
        super(CuminQueue, self).__init__(model, "queue", Queue, QueueStats)

        prop = CuminProperty(self, "name")
        prop.title = "Name"

        prop = CuminProperty(self, "durable")
        prop.title = "Durable?"
        prop.summary = True

        prop = CuminProperty(self, "autoDelete")
        prop.title = "Auto Delete?"
        prop.summary = True

        prop = CuminProperty(self, "exclusive")
        prop.title = "Exclusive?"
        prop.summary = True

        prop = CuminProperty(self, "arguments")
        prop.title = "Arguments"
        prop.prefix = "qpid."
        prop.escape = False

        stat = CuminStat(self, "consumerCount")
        stat.title = "Consumers"
        stat.unit = "consumer"
        stat.highlow = True

        stat = CuminStat(self, "bindingCount")
        stat.title = "Bindings"
        stat.unit = "binding"
        stat.highlow = True

        stat = CuminStat(self, "msgDepth")
        stat.title = "Message Depth"
        stat.highlow = True

        stat = CuminStat(self, "msgTotalEnqueues")
        stat.title = "Msgs. Enqueued"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "msgTotalDequeues")
        stat.title = "Msgs. Dequeued"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "byteDepth")
        stat.title = "Byte Depth"
        stat.unit = "byte"
        stat.highlow = True

        stat = CuminStat(self, "byteTotalEnqueues")
        stat.title = "Bytes Enqueued"
        stat.unit = "byte"
        stat.category = "io"

        stat = CuminStat(self, "byteTotalDequeues")
        stat.title = "Bytes Dequeued"
        stat.unit = "byte"
        stat.category = "io"

        stat = CuminStat(self, "unackedMessages")
        stat.title = "Msgs. Unacked"
        stat.unit = "message"
        stat.category = "io"
        stat.highlow = True

        stat = CuminStat(self, "messageLatencyMin")
        stat.title = "Min. Msg. Latency"
        stat.unit = "nanosecond"
        stat.category = "io"

        stat = CuminStat(self, "messageLatencyMax")
        stat.title = "Max. Msg. Latency"
        stat.unit = "nanosecond"
        stat.category = "io"

        stat = CuminStat(self, "messageLatencyAverage")
        stat.title = "Avg. Msg. Latency"
        stat.unit = "nanosecond"
        stat.category = "io"

        # Transactional

        stat = CuminStat(self, "msgTxnEnqueues")
        stat.title = "Msgs. Enqueued"
        stat.unit = "message"
        stat.category = "io.txn"

        stat = CuminStat(self, "msgTxnDequeues")
        stat.title = "Msgs. Dequeued"
        stat.unit = "message"
        stat.category = "io.txn"

        stat = CuminStat(self, "byteTxnEnqueues")
        stat.title = "Bytes Enqueued"
        stat.unit = "byte"
        stat.category = "io.txn"

        stat = CuminStat(self, "byteTxnDequeues")
        stat.title = "Bytes Dequeued"
        stat.unit = "byte"
        stat.category = "io.txn"

        stat = CuminStat(self, "enqueueTxnStarts")
        stat.title = "Enq. Trans. Started"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "enqueueTxnCommits")
        stat.title = "Enq. Trans. Committed"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "enqueueTxnRejects")
        stat.title = "Enq. Trans. Rejected"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "enqueueTxnCount")
        stat.title = "Enq. Trans. Pending"
        stat.unit = "transaction"
        stat.category = "txn"
        stat.highlow = True

        stat = CuminStat(self, "dequeueTxnStarts")
        stat.title = "Deq. Trans. Started"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "dequeueTxnCommits")
        stat.title = "Deq. Trans. Committed"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "dequeueTxnRejects")
        stat.title = "Deq. Trans. Rejected"
        stat.unit = "transaction"
        stat.category = "txn"

        stat = CuminStat(self, "dequeueTxnCount")
        stat.title = "Deq. Trans. Pending"
        stat.unit = "transaction"
        stat.category = "txn"
        stat.highlow = True

        # Durable

        stat = CuminStat(self, "msgPersistEnqueues")
        stat.title = "Msgs. Enqueued"
        stat.unit = "message"
        stat.category = "io.durable"

        stat = CuminStat(self, "msgPersistDequeues")
        stat.title = "Msgs. Dequeued"
        stat.unit = "message"
        stat.category = "io.durable"

        stat = CuminStat(self, "bytePersistEnqueues")
        stat.title = "Bytes Enqueued"
        stat.unit = "byte"
        stat.category = "io.durable"

        stat = CuminStat(self, "bytePersistDequeues")
        stat.title = "Bytes Dequeued"
        stat.unit = "byte"
        stat.category = "io.durable"

    def init(self):
        super(CuminQueue, self).init()

        self.frame = self.model.frame.messaging.broker.queue

    def get_title(self, session):
        return "Queue"

    def get_icon_href(self, session):
        return "resource?name=queue-36.png"

class CuminExchange(RemoteClass):
    def __init__(self, model):
        super(CuminExchange, self).__init__(model, "exchange",
                                            Exchange, ExchangeStats)

        prop = self.NameProperty(self, "name")
        prop.title = "Name"

        prop = CuminProperty(self, "type")
        prop.title = "Type"
        prop.summary = True

        prop = CuminProperty(self, "durable")
        prop.title = "Durable?"
        prop.summary = True

        prop = CuminProperty(self, "arguments")
        prop.title = "Arguments"
        prop.prefix = "qpid."
        prop.escape = False

        stat = CuminStat(self, "producerCount")
        stat.title = "Producers"
        stat.unit = "producer"
        stat.highlow = True

        stat = CuminStat(self, "bindingCount")
        stat.title = "Bindings"
        stat.unit = "binding"
        stat.highlow = True

        stat = CuminStat(self, "msgReceives")
        stat.title = "Msgs. Received"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "msgRoutes")
        stat.title = "Msgs. Routed"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "msgDrops")
        stat.title = "Msgs. Dropped"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "byteReceives")
        stat.title = "Bytes Received"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "byteRoutes")
        stat.title = "Bytes Routed"
        stat.unit = "message"
        stat.category = "io"

        stat = CuminStat(self, "byteDrops")
        stat.title = "Bytes Dropped"
        stat.unit = "message"
        stat.category = "io"

    def init(self):
        super(CuminExchange, self).init()

        self.frame = self.model.frame.messaging.broker.exchange

    def get_title(self, session):
        return "Exchange"

    def get_icon_href(self, session):
        return "resource?name=exchange-36.png"

    def get_object_title(self, session, object):
        name = super(CuminExchange, self).get_object_name(object)

        if name:
            return super(CuminExchange, self).get_object_title(session, object)
        else:
            return self.get_object_name(object)

    def get_object_name(self, object):
        return object.name and object.name or "Default Exchange"

    class NameProperty(CuminProperty):
        def value(self, session, object):
            value = getattr(object, self.name, None)
            return value and value or "Default Exchange"
            
class CuminBinding(RemoteClass):
    def __init__(self, model):
        super(CuminBinding, self).__init__(model, "binding",
                                           Binding, BindingStats)

        prop = CuminProperty(self, "bindingKey")
        prop.title = "Binding Key"

        stat = CuminStat(self, "msgMatched")
        stat.title = "Msgs. Matched"
        stat.unit = "message"

    def get_title(self, session):
        return "Binding"

    def get_object_name(self, binding):
        return "between %s and %s" % (binding.exchange.name, binding.queue.name)

class CuminRoute(RemoteClass):
    def __init__(self, model):
        super(CuminRoute, self).__init__ \
            (model, "route", Bridge, BridgeStats)

        prop = CuminProperty(self, "key")
        prop.title = "Route Key"

        prop = CuminProperty(self, "tag")
        prop.title = "Tag"

        prop = CuminProperty(self, "excludes")
        prop.title = "Excludes"

        prop = CuminProperty(self, "durable")
        prop.title = "Durable"

        prop = CuminProperty(self, "dest")
        prop.title = "Exchange"

    def get_title(self, session):
        return "Route"

    def get_object_name(self, route):
        return route.src

class CuminConnection(RemoteClass):
    def __init__(self, model):
        super(CuminConnection, self).__init__(model, "connection",
                                              ClientConnection,
                                              ClientConnectionStats)

        prop = CuminProperty(self, "address")
        prop.title = "Address"

        prop = CuminProperty(self, "SystemConnection")
        prop.title = "System Connection?"

        prop = CuminProperty(self, "authIdentity")
        prop.title = "Auth Identity?"

        prop = CuminProperty(self, "federationLink")
        prop.title = "Federation Link"

        stat = CuminStat(self, "closing")
        stat.title = "Closing Down"
        stat.category = "general"

        stat = CuminStat(self, "bytesFromClient")
        stat.title = "Bytes Sent"
        stat.unit = "byte"
        stat.category = "io"

        stat = CuminStat(self, "bytesToClient")
        stat.title = "Bytes Received"
        stat.unit = "byte"
        stat.category = "io"

        stat = CuminStat(self, "framesFromClient")
        stat.title = "Frames Sent"
        stat.unit = "frame"
        stat.category = "io"

        stat = CuminStat(self, "framesToClient")
        stat.title = "Frames Received"
        stat.unit = "frame"
        stat.category = "io"

    def init(self):
        super(CuminConnection, self).init()

        self.frame = self.model.frame.messaging.broker.connection

    def get_title(self, session):
        return "Connection"

    def get_icon_href(self, session):
        return "resource?name=client-36.png"

    def get_object_name(self, conn):
        return conn.address

# class CuminSession(RemoteClass):
#     def __init__(self, model):
#         super(CuminSession, self).__init__(model, "session",
#                                            Session, SessionStats)

#         prop = CuminProperty(self, "name")
#         prop.title = "Name"

#         prop = CuminProperty(self, "channelId")
#         prop.title = "Channel ID"

#         prop = CuminProperty(self, "detachedLifespan")
#         prop.title = "Detached Lifespan"

#         stat = CuminStat(self, "expireTime")
#         stat.title = "Expiration"
#         stat.category = "general"

#         stat = CuminStat(self, "framesOutstanding")
#         stat.title = "Frames Outstanding"
#         stat.unit = "frame"
#         stat.category = "general"

#         stat = CuminStat(self, "attached")
#         stat.title = "Attached"
#         stat.category = "general"

#     def get_title(self, session):
#         return "Session"

class CuminLink(RemoteClass):
    def __init__(self, model):
        super(CuminLink, self).__init__(model, "link", Link, LinkStats)

        prop = CuminProperty(self, "useSsl")
        prop.title = "Use SSL"
        prop.summary = True

        prop = CuminProperty(self, "durable")
        prop.title = "Durable"
        prop.summary = True

        stat = CuminStat(self, "closing")
        stat.title = "Closing Down"

        stat = CuminStat(self, "framesFromPeer")
        stat.title = "Frames from Peer"
        stat.unit = "frame"
        stat.category = "io"

        stat = CuminStat(self, "framesToPeer")
        stat.title = "Frames to Peer"
        stat.unit = "frame"
        stat.category = "io"

        stat = CuminStat(self, "bytesFromPeer")
        stat.title = "Bytes from Peer"
        stat.unit = "byte"
        stat.category = "io"

        stat = CuminStat(self, "bytesToPeer")
        stat.title = "Bytes to Peer"
        stat.unit = "byte"
        stat.category = "io"

    def get_title(self, session):
        return "Broker Link"

    def get_object_name(self, link):
        return "%s:%d" % (link.host, link.port)

class CuminBrokerStoreModule(RemoteClass):
    def __init__(self, model):
        super(CuminBrokerStoreModule, self).__init__ \
            (model, "store", Store, StoreStats)

        prop = CuminProperty(self, "location")
        prop.title = "Location"

class CuminJournal(RemoteClass):
    def __init__(self, model):
        super(CuminJournal, self).__init__(model, "journal",
                                           Journal, JournalStats)

        prop = CuminProperty(self, "name")
        prop.title = "Name"
        prop.summary = True

        prop = CuminProperty(self, "directory")
        prop.title = "Directory"

        stat = CuminStat(self, "initialFileCount")
        stat.title = "Initial File Count"
        stat.unit = "file"
        stat.category = "io.journal"

        stat = CuminStat(self, "dataFileSize")
        stat.title = "Data File Size"
        stat.unit = "byte"
        stat.category = "io.journal"

        stat = CuminStat(self, "recordDepth")
        stat.title = "Record Depth"
        stat.unit = "record"
        stat.category = "io.journal"

        stat = CuminStat(self, "recordEnqueues")
        stat.title = "Record Enqueues"
        stat.unit = "record"
        stat.category = "io.journal"

        stat = CuminStat(self, "outstandingAIOs")
        stat.title = "Outstanding AIOs"
        stat.unit = "aio"
        stat.category = "io.journal"

        stat = CuminStat(self, "freeFileCount")
        stat.title = "Free Files"
        stat.unit = "file"
        stat.category = "io.journal"

        stat = CuminStat(self, "availableFileCount")
        stat.title = "Avail. Files"
        stat.unit = "file"
        stat.category = "io.journal"

        stat = CuminStat(self, "writeWaitFailures")
        stat.title = "Write Wait Failures"
        stat.unit = "failure"
        stat.category = "io.journal"

        stat = CuminStat(self, "writeBusyFailures")
        stat.title = "Write Busy Failures"
        stat.unit = "failure"
        stat.category = "io.journal"

        stat = CuminStat(self, "readRecordCount")
        stat.title = "Read Records"
        stat.unit = "record"
        stat.category = "io.journal"

        stat = CuminStat(self, "readBusyFailures")
        stat.title = "Read Busy Failures"
        stat.unit = "failure"
        stat.category = "io.journal"

        stat = CuminStat(self, "writePageCacheDepth")
        stat.title = "Write Page Cache Depth"
        stat.unit = "page"
        stat.category = "io.journal"

        stat = CuminStat(self, "readPageCacheDepth")
        stat.title = "Read Page Cache Depth"
        stat.unit = "page"
        stat.category = "io.journal"

class CuminBrokerAclModule(RemoteClass):
    def __init__(self, model):
        super(CuminBrokerAclModule, self).__init__ \
            (model, "store", Acl, AclStats)

        prop = CuminProperty(self, "policyFile")
        prop.title = "Policy File"

        prop = CuminProperty(self, "enforcingAcl")
        prop.title = "Enforcing?"

        prop = CuminProperty(self, "transferAcl")
        prop.title = "Transfer?"

        prop = CuminProperty(self, "lastAclLoad")
        prop.title = "Last Loaded"

        stat = CuminStat(self, "aclDenyCount")
        stat.title = "Denials"
        stat.unit = "denial"

class CuminBrokerClusterModule(RemoteClass):
    def __init__(self, model):
        super(CuminBrokerClusterModule, self).__init__ \
            (model, "broker_cluster_module", Cluster, ClusterStats)

        prop = CuminProperty(self, "clusterName")
        prop.title = "Name"

        prop = CuminProperty(self, "clusterID")
        prop.title = "ID"

        prop = CuminProperty(self, "publishedURL")
        prop.title = "URL"

        prop = CuminProperty(self, "clusterSize")
        prop.title = "Size"

        prop = CuminProperty(self, "status")
        prop.title = "Status"

        prop = CuminProperty(self, "members")
        prop.title = "Members"

class CuminLimit(CuminClass):
    def __init__(self, model):
        super(CuminLimit, self).__init__ \
            (model, "limit", None)

        prop = CuminProperty(self, "name")
        prop.title = "Name"

        prop = CuminProperty(self, "curr")
        prop.summary = True
        prop.title = "Current"

        prop = CuminProperty(self, "max")
        prop.summary = True
        prop.title = "Maximum"

    def init(self):
        super(CuminLimit, self).init()

        #self.frame = self.model.frame.grid.pool.limit

    def get_title(self, session):
        return "Concurrency Limit"

    def get_object_title(self, session, limit):
        title = self.get_title(session)
        return "%s '%s'" % (title, limit)

class JobMetaData(MetaData):
    def __init__(self, name):
        super(JobMetaData, self).__init__(name)

        ### Main Group
        prop = AdProperty(self, "Owner")
        prop.group = "Main"
        prop.description = "The submitter of the job"
        prop.writable = False

        prop = AdProperty(self, "GlobalJobId")
        prop.group = "Main"
        prop.description = "Unique job id"
        prop.writable = False

        prop = AdProperty(self, "Submission")
        prop.group = "Main"
        prop.description = "Submission name"
        prop.writable = False

        prop = self.JobUniverse(self, "JobUniverse")
        prop.group = "Main"
        prop.title = "Job Universe"
        prop.writable = False
        prop.renderer = prop.render_universe

        ### Condor Info Group
        prop = AdProperty(self, "CondorVersion")
        prop.group = "Condor Info"
        prop.writable = False

        prop = AdProperty(self, "CondorPlatform")
        prop.group = "Condor Info"
        prop.writable = False

        ### Command Info Group
        prop = AdProperty(self, "Args")
        prop.description = "Arguments passed to job Cmd"
        prop.group = "Command Info"

        prop = AdProperty(self, "Cmd")
        prop.description = "Command that will run the job"
        prop.group = "Command Info"

        prop = AdProperty(self, "In")
        prop.description = "The file where the job's standard input is read"
        prop.group = "Command Info"

        prop = AdProperty(self, "Out")
        prop.description = "The file where the job's standard output is written"
        prop.example = "'/dev/null' or '~/logs/'"
        prop.group = "Command Info"

        prop = AdProperty(self, "Err")
        prop.description = "The file where the job's errors are written"
        prop.group = "Command Info"

        prop = AdProperty(self, "Iwd")
        prop.description = "Command Input Working Directory"
        prop.group = "Command Info"
        prop.title = "Working Directory"

        prop = AdProperty(self, "UserLog")
        prop.description = "Log file"
        prop.group = "Command Info"

        ######## Job Status Info
        prop = self.JobStatusProperty(self, "JobStatus")
        prop.description = "The current job status"
        prop.group = "Job Status Info"
        prop.renderer = prop.render_status
        prop.title = "Job Status"
        prop.writable = False

        prop = AdProperty(self, "HoldReasonCode")
        prop.group = "Job Status Info"
        prop.title = "Hold Reason Code"
        prop.writable = False

        prop = self.JobStatusProperty(self, "ExitStatus")
        prop.description = "Status when job completes"
        prop.group = "Job Status Info"
        prop.renderer = prop.render_status
        prop.title = "Exit Status"
        prop.writable = False

        prop = self.JobStatusProperty(self, "LastJobStatus")
        prop.description = ""
        prop.group = "Job Status Info"
        prop.renderer = prop.render_status
        prop.writable = False

        ######## Other
        prop = AdProperty(self, "ProcId")
        prop.description = "The id of the job within its cluster. Proc Ids are unique within a cluster."
        prop.group = "Other"
        prop.title = "Proc Id"
        prop.writable = False

        prop = AdProperty(self, "BufferBlockSize")
        prop.example = "32768"
        prop.group = "Other"

        prop = AdProperty(self, "ClusterId")
        prop.description = "The id of the cluster the job belongs to"
        prop.group = "Other"
        prop.title = "Cluster ID"
        prop.writable = False

        ###### Dates
        prop = DateAdProperty(self, "QDate")
        prop.description = "When the job was submitted"
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "LastVacateTime")
        prop.group = "Dates"
        prop.writable = False
        
        prop = DateAdProperty(self, "JobStartDate")
        prop.group = "Main"
        prop.writable = False

        prop = DateAdProperty(self, "JobCurrentStartDate")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "JobLastStartDate")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "LastSuspensionTime")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "ShadowBday")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "LastJobLeaseRenewal")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "EnteredCurrentStatus")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "CommittedTime")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "LastMatchTime")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "ScheddBday")
        prop.group = "Dates"
        prop.writable = False

        prop = DateAdProperty(self, "LastRejMatchTime")
        prop.group = "Dates"
        prop.writable = False

        # other
        prop = AdProperty(self, "User")
        prop.group = "Other"
        prop.writable = False

        prop = AdProperty(self, "MinHosts")
        prop.group = "Other"
        prop.writable = False

        prop = AdProperty(self, "RemoteUserCpu")
        prop.group = "Other"
        prop.writable = False

        prop = AdProperty(self, "DiskUsage")
        prop.group = "Other"
        prop.writable = False

        prop = AdProperty(self, "ImageSize")
        prop.group = "Other"
        prop.writable = False

    class JobStatusProperty(AdProperty):
        def render_status(self, session, status):
            return JobStatusInfo.get_status_string(status)

    class JobUniverse(AdProperty):
        universes = {None: "Default",
                    5: "Vanilla",
                    7: "Scheduler",
                    9: "Grid",
                    10: "Java",
                    11: "Parallel",
                    12: "Local",
                    13: "VM"}

        def render_universe(self, session, value):
            try:
                return self.universes[value]
            except KeyError:
                return "Unknown (%s)" % str(value)

class GetStartedAction(CuminAction):
    def get_xml_response(self, session, object, *args):
        updateTime = object.statsCurr and object.statsCurr.qmfUpdateTime \
            or object.qmfUpdateTime
        delta = secs(updateTime) - secs(datetime.now())
        conf = "<age>%i</age>" % -delta
        rect = "<updatetime>%s</updatetime>" % fmt_duration(delta)

        return "%s%s" % (conf, rect)

class QmfException(Exception):
    def __init__(self, value):
        self.message = value

    def __str__(self):
        return repr(self.message)

class QmfCall(object):
    def __init__(self, app, default=None, timeout=5):
        self.app = app
        self.data = default
        self.got_data = False
        self.error = False
        self.status = None
        self.timeout = timeout

    def get_completion(self):
        def completion(status, data):
            self.status = status
            if (status == 0) or (status == "OK"):
                self.data = data
                self.got_data = True
            else:
                self.error = QmfException(status)
                msg = "QMF call returned with status of %s" % str(status)
                log.error(msg)
        return completion

    def do_wait(self):
        wait(self.done, timeout=self.timeout)
        return self

    def done(self):
        return self.got_data or self.error

    def execute(self, obj, method_name, *args):
        session = self.app.session
        try:
            session.call_method(self.get_completion(), obj, method_name, args)
        except Exception, e:
            self.error = e
            log.exception(e)

        results = self.do_wait()
        if not results.got_data and not results.error:
            results.error = QmfException("Request timed out")
            msg = "QMF call %s timed out" % method_name
            log.error(msg)

        return results

class QmfCallSet(object):
    def __init__(self, app):
        self.app = app
        self.calls = dict()
        self.timeout = 5

    def add_call(self, key, default=None):
        call = QmfCall(self.app, default)
        self.calls[key] = call
        return call

    def do_wait(self):
        def predicate(calls):
            done = 0
            for call in calls:
                if calls[call].done():
                    done += 1
            return done == len(calls)

        wait(predicate, timeout=self.timeout, args=self.calls)
        return self.calls

class FetchRawConfigSet(QmfCallSet):
    def execute(self, negotiator, groups, prepend=""):
        default = {'Value': 0}
        session = self.app.session
        for group in groups:
            call = self.add_call(group, default)

            try:
                session.call_method(call.get_completion(), negotiator, "GetRawConfig", (prepend+group,))
            except Exception, e:
                call.error = e
                log.exception(e)

        return self.do_wait()

class FetchJobAd(QmfCall):
    def __init__(self, app):
        super(FetchJobAd, self).__init__(app)
        self.data = {'JobAd': {"":{"VALUE":"", "TYPE":0}}}

class FetchJobOutput(QmfCall):
    def __init__(self, app):
        super(FetchJobOutput, self).__init__(app)
        self.data = {'Data': ""}

    def execute(self, job_server, jobId, file, start, end):
        session = self.app.session
        try:
            session.call_method(self.get_completion(), job_server, "Fetch", (jobId, file, start, end))
        except Exception, e:
            self.error = e
            log.exception(e)
        return self.do_wait()

class CuminScheduler(RemoteClass):
    def __init__(self, model):
        super(CuminScheduler, self).__init__(model, "scheduler",
                                             Scheduler, SchedulerStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        prop = CuminProperty(self, "JobQueueBirthdate")
        prop.title = "Job Queue Birthdate"

        prop = CuminProperty(self, "MaxJobsRunning")
        prop.title = "Max Jobs Running"

        prop = CuminProperty(self, "MyAddress")
        prop.title = "Contact Address"

        prop = CuminProperty(self, "PublicNetworkIpAddr")
        prop.title = "Public IP Address"

        prop = CuminProperty(self, "Machine")
        prop.title = "Machine"
        prop.summary = True

        prop = CuminProperty(self, "System")
        prop.title = "System"

        stat = CuminStat(self, "NumUsers")
        stat.title = "Users"

        stat = CuminStat(self, "TotalRunningJobs")
        stat.title = "Running Jobs"

        stat = CuminStat(self, "TotalIdleJobs")
        stat.title = "Idle Jobs"

        stat = CuminStat(self, "TotalHeldJobs")
        stat.title = "Held Jobs"

        stat = CuminStat(self, "TotalJobAds")
        stat.title = "Job Ads"

        stat = CuminStat(self, "TotalRemovedJobs")
        stat.title = "Removed Jobs"

        action = GetStartedAction(self, "GetStarted")
        action.navigable = False

    def init(self):
        super(CuminScheduler, self).init()

        # self.frame = self.model.frame.grid.pool.scheduler

    def get_title(self, session):
        return "Scheduler"

    def get_object_name(self, sched):
        return sched.Name

class CuminSubmission(RemoteClass):
    def __init__(self, model):
        super(CuminSubmission, self).__init__(model, "submission",
                                              Submission, SubmissionStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        stat = CuminStat(self, "Idle")
        stat.title = "Idle Jobs"

        stat = CuminStat(self, "Running")
        stat.title = "Running Jobs"

        stat = CuminStat(self, "Removed")
        stat.title = "Removed Jobs"

        stat = CuminStat(self, "Completed")
        stat.title = "Completed Jobs"

        stat = CuminStat(self, "Held")
        stat.title = "Held Jobs"

    def get_title(self, session):
        return "Submission"

    def get_object_name(self, sub):
        return sub.Name

class CuminSubmitter(RemoteClass):
    def __init__(self, model):
        super(CuminSubmitter, self).__init__(model, "submitter",
                                             Submitter, SubmitterStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        stat = CuminStat(self, "RunningJobs")
        stat.title = "Running Jobs"

        stat = CuminStat(self, "IdleJobs")
        stat.title = "Idle Jobs"

        stat = CuminStat(self, "HeldJobs")
        stat.title = "Held Jobs"

    def init(self):
        super(CuminSubmitter, self).init()

        #self.frame = self.model.frame.grid.pool.scheduler.submitter

    def get_title(self, session):
        return "Submitter"

    def get_object_name(self, sub):
        return sub.Name

    def show_object(self, session, sub):
        pool = self.model.get_main_pool()

        assert pool

        self.model.frame.grid.pool.set_object(session, pool)

        return self.frame.show_object(session, sub)

class CuminCollector(RemoteClass):
    def __init__(self, model):
        super(CuminCollector, self).__init__(model, "collector",
                                             Collector, CollectorStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        prop = CuminProperty(self, "PublicNetworkIpAddr")
        prop.title = "IP Address"

        action = GetStartedAction(self, "GetStarted")
        action.navigable = False

        stat = CuminStat(self, "RunningJobs")
        stat.title = "Running Jobs"

        stat = CuminStat(self, "IdleJobs")
        stat.title = "Idle Jobs"

        stat = CuminStat(self, "HostsClaimed")
        stat.title = "Claimed Slots"

        stat = CuminStat(self, "HostsUnclaimed")
        stat.title = "Unclaimed Slots"

        stat = CuminStat(self, "HostsOwner")
        stat.title = "Owner Slots"

        stat = CuminStat(self, "HostsTotal")
        stat.title = "Total Slots"

    def init(self):
        super(CuminCollector, self).init()

        #self.frame = self.model.frame.grid.pool.collector

    def get_title(self, session):
        return "Collector"

    def get_object_name(self, coll):
        return coll.Name

class CuminNegotiator(RemoteClass):
    def __init__(self, model):
        super(CuminNegotiator, self).__init__(model, "negotiator",
                                              Negotiator, NegotiatorStats)

        prop = CuminProperty(self, "Name")
        prop.title = "Name"

        prop = CuminProperty(self, "Machine")
        prop.title = "Machine"

        prop = CuminProperty(self, "MyAddress")
        prop.title = "Address"

        prop = CuminProperty(self, "DaemonStartTime")
        prop.title = "Start Time"

        stat = CuminStat(self, "MonitorSelfAge")
        stat.title = "Age"
        stat.category = "general"

        stat = CuminStat(self, "MonitorSelfCPUUsage")
        stat.title = "CPU Usage"
        stat.category = "general"

        stat = CuminStat(self, "MonitorSelfImageSize")
        stat.title = "Image Size"
        stat.category = "general"

        stat = CuminStat(self, "MonitorSelfRegisteredSocketCount")
        stat.title = "Registered Socket Count"
        stat.category = "general"

        stat = CuminStat(self, "MonitorSelfResidentSetSize")
        stat.title = "Resident Set Size"
        stat.category = "general"

        action = GetStartedAction(self, "GetStarted")
        action.navigable = False

    def init(self):
        super(CuminNegotiator, self).init()

        #self.frame = self.model.frame.grid.pool.negotiator

    def get_title(self, session):
        return "Negotiator"

    def get_object_name(self, neg):
        return neg.Name

class CuminSubject(CuminClass):
    def __init__(self, model):
        super(CuminSubject, self).__init__(model, "subject", Subject)

    def get_title(self, session):
        return "Subject"

class Pool(object):
    def __init__(self, id):
        self.id = id

    def get_collector(self):
        for collector in Collector.selectBy(Pool=self.id):
            return collector

    def get_grid(self):
        for grid in Grid.selectBy(Pool=self.id):
            return grid

    def get_negotiator(self):
        for negotiator in Negotiator.selectBy(Pool=self.id):
            return negotiator

class ObjectStore(object):
    def __init__(self, model):
        self.model = model
        self.data = None
        self.status = None
        self.exception = None

        self.update_thread = self.UpdateThread(self)

    def start_updates(self):
        self.update_thread.start()

    def update(self):
        pass

    def delete(self):
        self.model = None

    class UpdateThread(Thread):
        def __init__(self, store):
            Thread.__init__(self)

            self.store = store
            self.setDaemon(True)

        def run(self):
            conn = self.store.model.app.database.get_connection()
            cursor = conn.cursor()

            try:
                for i in range(20):
                    try:
                        self.store.update(cursor)
                        self.store.exception = None
                    except Exception, e:
                        log.exception(e)
                        self.store.exception = e

                    sleep(30)
            finally:
                conn.close()

            self.store.delete()

class UpdateTimedOut(Exception):
    pass

class NegotiatorLimitStore(ObjectStore):
    def __init__(self, model, negotiator):
        super(NegotiatorLimitStore, self).__init__(model)

        self.negotiator = negotiator

    def update(self, cursor):
        def completion(status, data):
            self.status = status
            try:
                self.data = data["Limits"]
            except KeyError:
                pass

        self.model.app.session.call_method \
            (completion, self.negotiator, "GetLimits", ())

    def delete(self):
        del self.model.limits_by_negotiator[self.negotiator._qmf_agent_id]

        super(NegotiatorLimitStore, self).delete()

class SubmissionJobSummaryStore(ObjectStore):
    def __init__(self, model, submission):
        super(SubmissionJobSummaryStore, self).__init__(model)

        self.submission = submission

    def update(self, cursor):
        def completion(status, data):
            self.status = status
            try:
                self.data = data["Jobs"]
            except KeyError:
                pass

        self.model.app.session.call_method \
            (completion, self.submission, "GetJobSummaries", ())

    def delete(self):
        del self.model.job_summaries_by_submission[self.submission._id]

        super(SubmissionJobSummaryStore, self).delete()

class NegotiatorGroupNamesStore(ObjectStore):
    def __init__(self, model, negotiator):
        super(NegotiatorGroupNamesStore, self).__init__(model)

        self.negotiator = negotiator

    def update(self, cursor):
        def completion(status, data):
            self.status = status
            try:
                self.data = data["Value"]
            except KeyError:
                pass

        self.model.app.session.call_method \
            (completion, self.negotiator, "GetRawConfig", ("GROUP_NAMES",))

    def delete(self):
        del self.model.group_names_by_negotiator[self.negotiator._qmf_agent_id]

        super(NegotiatorGroupNamesStore, self).delete()

class NegotiatorGroupConfigValuesStore(ObjectStore):
    def __init__(self, model, negotiator, groups, config):
        super(NegotiatorGroupConfigValuesStore, self).__init__(model)

        self.negotiator = negotiator
        self.configs = dict()

        self.configs[config] = list(groups)
        self.data = dict()

    def add_group_config(self, group, config):
        added = 0

        try:
            groups = self.configs[config]
            if not group in groups:
                groups.append(group)
                added = 1
        except KeyError:
            self.configs[config] = [group]
            added = 1

        return added

    def update(self, cursor):
        for config in self.configs:
            action = FetchRawConfigSet(self.model.app)
            raw_configs = action.execute(self.negotiator, self.configs[config], config+"_")
            self.data[config] = raw_configs

    def delete(self):
        del self.model.group_config_values_by_negotiator[self.negotiator._qmf_agent_id]

        super(NegotiatorGroupConfigValuesStore, self).delete()
