from wooly import *
from wooly.datatable import *
from wooly.forms import *
from wooly.widgets import *

from objectframe import *
from objecttask import *
from sqladapter import *
from qmfadapter import *
from util import *

strings = StringCatalog(__file__)

class ObjectTable(DataTable):
    def __init__(self, app, name, cls):
        super(ObjectTable, self).__init__(app, name)

        assert isinstance(cls, RosemaryClass), cls

        self.cls = cls

        self.update_enabled = True
        self.ascending.default = True

        # (RosemaryAttribute this, RosemaryAttribute that, Attribute object)
        self.filter_specs = list()

    def init(self):
        if not self.adapter:
            self.adapter = ObjectSqlAdapter(self.app, self.cls)

        super(ObjectTable, self).init()

        for this, that, fobj in self.filter_specs:
            self.adapter.add_value_filter(this)

        if self.sort.default is None:
            for col in self.columns:
                if col.sortable:
                    self.sort.default = col.name
                    break

    def add_attribute_column(self, attr):
        assert isinstance(attr, RosemaryAttribute), attr

        col = ObjectTableColumn(self.app, attr.name, attr)

        self.add_column(col)

        return col

    def add_filter(self, attribute, this, that=None):
        if not that:
            that = this

        assert isinstance(attribute, Attribute), attribute
        assert isinstance(this, RosemaryAttribute), this
        assert isinstance(that, RosemaryAttribute), that

        self.filter_specs.append((this, that, attribute))

    def add_reference_filter(self, attribute, ref):
        assert isinstance(ref, RosemaryReference), ref

        this = ref
        that = ref.that_cls._id

        self.add_filter(attribute, this, that)

    def get_data_values(self, session):
        values = dict()

        for this, that, fobj in self.filter_specs:
            obj = fobj.get(session)
            values[this.name] = getattr(obj, that.name)

        return values

    def render_title(self, session):
        return "%ss" % self.cls._title

class ObjectTableColumn(DataTableColumn):
    def __init__(self, app, name, attr):
        super(ObjectTableColumn, self).__init__(app, name)

        self.attr = attr

    def init(self):
        super(ObjectTableColumn, self).init()

        assert self.table.adapter, self.table

        try:
            self.field = self.table.adapter.fields_by_attr[self.attr]
        except KeyError:
            # XXX do this instead:
            #cls = self.table.adapter.default_field_cls
            #self.field = cls(self.table.adapter, self.attr)

            if isinstance(self.table.adapter, ObjectQmfAdapter):
                self.field = ObjectQmfField(self.table.adapter, self.attr)
            else:
                self.field = ObjectSqlField(self.table.adapter, self.attr)

class ObjectSelector(ObjectTable, Form):
    def __init__(self, app, name, cls):
        super(ObjectSelector, self).__init__(app, name, cls)

        self.init_ids(app, cls)

        self.switches = ObjectSelectorSwitches(app, "switches")
        self.add_child(self.switches)

        self.filters = ObjectSelectorFilters(app, "filters")
        self.add_child(self.filters)

        self.buttons = ObjectSelectorButtons(app, "buttons")
        self.add_child(self.buttons)

        self.links = ObjectSelectorLinks(app, "links")
        self.add_child(self.links)

        self.tasks = list()

    def init(self):
        super(ObjectSelector, self).init()

        for task in self.tasks:
            task.init()

        for task in self.tasks:
            button = ObjectTaskButton(self.app, task)
            self.buttons.add_child(button)
            button.init()

    def init_ids(self, app, cls):
        item = IntegerParameter(app, "item")

        self.ids = ListParameter(app, "selection", item)
        self.add_parameter(self.ids)

        self.checkbox_column = ObjectCheckboxColumn \
            (app, "id", cls._id, self.ids)
        self.add_column(self.checkbox_column)

class ObjectCheckboxColumn(ObjectTableColumn):
    def __init__(self, app, name, attr, selection):
        super(ObjectCheckboxColumn, self).__init__(app, name, attr)

        self.selection = selection

        self.header = CheckboxColumnHeader(app, "header")
        self.replace_child(self.header)

        self.cell = CheckboxColumnCell(app, "cell", selection)
        self.replace_child(self.cell)

        self.cell.input = ObjectCheckboxColumnInput(app, "input", selection)
        self.cell.replace_child(self.cell.input)

        self.sortable = False
        self.width = "1%"

    def render_cell_value(self, session, record):
        return record[self.field.index]

class ObjectCheckboxColumnInput(CheckboxColumnInput):
    def render_onclick_attr(self, session, record):
        value = "cumin.clickTableCheckbox(this, '%s')" % \
            self.parent.parent.selection.path

        return "onclick=\"%s\"" % value

    def render_checked_attr(self, session, record):
        checks = self.get(session)
        return record[self.parent.parent.field.index] in checks \
            and "checked=\"checked\"" or ""

class ObjectLinkColumn(ObjectTableColumn, LinkColumn):
    def __init__(self, app, name, attr, id_attr, frame_path):
        super(ObjectLinkColumn, self).__init__(app, name, attr)

        self.id_attr = id_attr
        self.frame_path = frame_path

    def init(self):
        super(ObjectLinkColumn, self).init()
 
        try:
            self.id_field = self.table.adapter.fields_by_attr[self.id_attr]
        except KeyError:
            if isinstance(self.table.adapter, ObjectQmfAdapter):
                self.id_field = ObjectQmfField(self.table.adapter, self.id_attr)
            else:
                self.id_field = ObjectSqlField(self.table.adapter, self.id_attr)

    def render_cell_href(self, session, record):
        id = record[self.id_field.index]
        frame = self.page.page_widgets_by_path[self.frame_path]

        if isinstance(frame, ObjectFrame):
            return frame.get_href(session, id)
        else:
            return frame.get_href(session, Identifiable(id)) # XXX

    def render_cell_content(self, session, record):
        return record[self.field.index]

class ObjectSelectorControl(WidgetSet):
    def do_render(self, session):
        if len(self.children):
            return super(ObjectSelectorControl, self).do_render(session)

class ObjectSelectorSwitches(ObjectSelectorControl):
    pass

class ObjectSelectorFilters(ObjectSelectorControl):
    pass

class ObjectSelectorButtons(ObjectSelectorControl):
    pass

class ObjectSelectorLinks(ObjectSelectorControl):
    pass

class ObjectSelectorTask(ObjectTask):
    def __init__(self, app, selector):
        super(ObjectSelectorTask, self).__init__(app)

        self.cls = selector.cls

        self.selector = selector
        self.selector.tasks.append(self)

        self.form = None

    def init(self):
        super(ObjectSelectorTask, self).init()

        if not self.form:
            self.form = ObjectSelectorTaskForm(self.app, self.name, self)
            self.form.init()

    def get_title(self, session):
        pass

    def enter(self, session):
        ids = self.selector.ids.get(session)

        nsession = wooly.Session(self.app.form_page)

        self.form.ids.set(nsession, ids)
        self.form.return_url.set(nsession, session.marshal())
        self.form.show(nsession)

        self.do_enter(nsession, session)

        return nsession

    def do_enter(self, session, osession):
        pass

    def invoke(self, session, selection, *args):
        for item in selection:
            invoc = self.start(session, item)

            try:
                self.do_invoke(invoc, item, *args)
            except Exception, e:
                invoc.exception = e
                invoc.status = invoc.FAILED
                invoc.end()

    def do_invoke(self, invoc, item, *args):
        pass

    def get_item_content(self, session, item):
        return item.get_formatted_value("name")

class ObjectSelectorTaskForm(FoldingFieldSubmitForm):
    def __init__(self, app, name, task):
        super(ObjectSelectorTaskForm, self).__init__(app, name)

        self.task = task

        self.cls = task.selector.cls

        item = IntegerParameter(app, "item")

        self.ids = ListParameter(app, "id", item)
        self.add_parameter(self.ids)

        self.selection = SessionAttribute(self, "selection")

        self.content = self.SelectionList(app, "fields")
        self.replace_child(self.content)

        self.app.form_page.modes.add_mode(self)

    def do_process(self, session):
        self.get_selection(session)
        super(ObjectSelectorTaskForm, self).do_process(session)

    def get_selection(self, session):
        selection = list()

        self.selection.set(session, selection)

        for id in self.ids.get(session):
            item = self.cls.get_object_by_id(session.cursor, id)
            selection.append(item)

    def process_submit(self, session):
        selection = self.selection.get(session)

        self.task.invoke(session, selection)
        self.task.exit_with_redirect(session)

    def render_title(self, session):
        return self.task.get_title(session)

    class SelectionList(ItemSet):
        def do_get_items(self, session):
            return self.parent.selection.get(session)

        def render_item_content(self, session, item):
            return self.parent.task.get_item_content(session, item)

        def render_item_class(self, session, item):
            return "item"
