from forms import *
from parameters import *
from util import *
from wooly import *

strings = StringCatalog(__file__)

class Table(Widget):
    def __init__(self, app, name):
        super(Table, self).__init__(app, name)

        self.header = TableHeader(app, "header")
        self.add_child(self.header)

        self.footer = TableFooter(app, "footer")
        self.add_child(self.footer)

        self.columns = list()

        self.row = TableRow(app, "row")
        self.add_child(self.row)

        self.sort = SymbolParameter(app, "sort")
        self.add_parameter(self.sort)

        self.ascending = BooleanParameter(app, "ascending")
        self.add_parameter(self.ascending)

    def add_column(self, column):
        self.add_child(column)

        self.columns.append(column)

    def insert_column(self, index, column):
        self.add_child(column)

        self.columns.insert(index, column)

    def get_data(self, session):
        return ()

    def get_visible_columns(self, session):
        return [x for x in self.columns if x.visible]

    def render_css(self, session):
        writer = Writer()

        for column in self.get_visible_columns(session):
            writer.write(column.css.render(session))

        return writer.to_string()

    def render_columns(self, session):
        writer = Writer()

        for column in self.get_visible_columns(session):
            writer.write(column.render(session))

        return writer.to_string()

    def render_colspan(self, session):
        return len(self.get_visible_columns(session))
            
    def render_rows(self, session):
        data = self.get_data(session)

        writer = Writer()

        for record in data:
            writer.write(self.row.render(session, record))

        return writer.to_string()

class TableChild(Widget):
    def init(self):
        super(TableChild, self).init()

        for anc in self.ancestors:
            if isinstance(anc, Table):
                self.table = anc

        assert self.table, "Not inside a table"

class TableColumn(TableChild):
    def __init__(self, app, name):
        super(TableColumn, self).__init__(app, name)

        self.css = TableColumnCss(app, "css")
        self.add_child(self.css)

        self.header = TableColumnHeader(app, "header")
        self.add_child(self.header)

        self.cell = TableColumnCell(app, "cell")
        self.add_child(self.cell)

        self.sortable = True
        self.visible = True
        self.width = None

    def render_class(self, session):
        tokens = list()
        tokens.append(self.name)

        if self.name == self.table.sort.get(session):
            tokens.append("selected")

        return " ".join(tokens)

    def render_width(self, session):
        width = self.width

        if not width:
            width = "auto"

        return width

    def render_text_align(self, session):
        return "left"

    def render_header_href(self, session):
        sort = self.table.sort.get(session)
        ascending = self.table.ascending.get(session)

        branch = session.branch()

        self.table.sort.set(branch, self.name)

        if sort == self.name:
            self.table.ascending.set(branch, not ascending)

        return branch.marshal()

    def render_header_content(self, session):
        pass

    def render_cell_content(self, session, record):
        pass
    
class TableColumnCss(TableChild):
    def render_class(self, session):
        return self.parent.name

    def render_text_align(self, session):
        return self.parent.render_text_align(session)

    def render_width(self, session):
        return self.parent.render_width(session)

class TableColumnHeader(TableChild):
    def render_class(self, session):
        return self.parent.render_class(session)

    def render_href(self, session):
        return self.parent.render_header_href(session)

    def render_content(self, session):
        return self.parent.render_header_content(session)

class TableColumnCell(TableChild):
    def render_class(self, session, record):
        return self.parent.render_class(session)

    def render_content(self, session, record):
        return self.parent.render_cell_content(session, record)

class TableHeader(TableChild):
    def render_colspan(self, session):
        return self.table.render_colspan(session)

    def render_headers(self, session):
        writer = Writer()

        for column in self.table.get_visible_columns(session):
            writer.write(column.header.render(session))

        return writer.to_string()

class TableFooter(TableChild):
    def render_colspan(self, session):
        return self.table.render_colspan(session)

class TableRow(TableChild):
    def render_cells(self, session, record):
        writer = Writer()

        for column in self.table.get_visible_columns(session):
            writer.write(column.cell.render(session, record))

        return writer.to_string()

class LinkColumn(TableColumn):
    def __init__(self, app, name):
        super(LinkColumn, self).__init__(app, name)

        self.cell = LinkColumnCell(app, "cell")
        self.replace_child(self.cell)

    def render_cell_href(self, session, record):
        pass

class LinkColumnCell(TableColumnCell):
    def render_href(self, session, record):
        return self.parent.render_cell_href(session, record)

class NewCheckboxColumn(TableColumn):
    def __init__(self, app, name):
        super(NewCheckboxColumn, self).__init__(app, name)

        self.selection = selection # XXX huh?

        self.header = CheckboxColumnHeader(app, "header")
        self.replace_child(self.header)

        self.cell = CheckboxColumnCell(app, "cell", selection)
        self.replace_child(self.cell)

        self.sortable = False
        self.width = "1%"

    def init(self):
        super(NewCheckboxColumn, self).init()

        assert self.selection, self

    def render_cell_value(self, session, record):
        pass

class CheckboxColumnHeader(TableColumnHeader):
    def render_name(self, session):
        return self.parent.selection.path

class CheckboxColumnCell(TableColumnCell):
    def __init__(self, app, name, selection):
        super(CheckboxColumnCell, self).__init__(app, name)

        self.input = CheckboxColumnInput(app, "input", selection)
        self.add_child(self.input)

    def render_content(self, session, record):
        return self.input.render(session, record)

class CheckboxColumnInput(CheckboxInput):
    def render_value(self, session, record):
        return self.parent.parent.render_cell_value(session, record)
